package org.rcisoft.sys.attendance.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.service.AttendanceSyncService;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;



import java.util.List;

/**
* Created by cy on 2022年11月28日 下午4:57:21.
*/
@RestController
@RequestMapping("/attendancesync")
public class AttendanceSyncController extends CyPaginationController<AttendanceSync> {


    @Autowired
    private AttendanceSyncService attendanceSyncServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:add')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-新增考勤同步表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加考勤同步表", notes="添加考勤同步表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid AttendanceSync attendanceSync, BindingResult bindingResult) {
        CyPersistModel data = attendanceSyncServiceImpl.persist(attendanceSync);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        attendanceSync);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:delete')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-删除考勤同步表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除考勤同步表", notes="逻辑删除考勤同步表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable String businessId,AttendanceSync attendanceSync) {
        attendanceSync.setBusinessId(businessId);
        CyPersistModel data = attendanceSyncServiceImpl.removeLogical(attendanceSync);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:delete')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-删除考勤同步表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除考勤同步表", notes="删除考勤同步表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable String businessId,AttendanceSync attendanceSync) {
        attendanceSync.setBusinessId(businessId);
        CyPersistModel data = attendanceSyncServiceImpl.remove(attendanceSync);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:update')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-修改考勤同步表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改考勤同步表", notes="修改考勤同步表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody AttendanceSync attendanceSync, BindingResult bindingResult) {
        CyPersistModel data = attendanceSyncServiceImpl.merge(attendanceSync);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        attendanceSync);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:query')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-查询考勤同步表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一考勤同步表", notes="查询单一考勤同步表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        attendanceSyncServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:list')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-查询考勤同步表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询考勤同步表集合", notes="查询考勤同步表集合")
    @GetMapping(value = "/queryAttendanceSyncs")
    public CyResult queryAttendanceSyncs(AttendanceSync attendanceSync) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            attendanceSyncServiceImpl.findAll(attendanceSync));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:endanceSync:list')")
    @CyOpeLogAnno(title = "system-考勤同步表管理-查询考勤同步表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询考勤同步表集合", notes="分页查询考勤同步表集合")
    @GetMapping(value = "/queryAttendanceSyncByPagination")
    public CyGridModel listByPagination(AttendanceSync attendanceSync) {
        attendanceSyncServiceImpl.findAllByPagination(getPaginationUtility(), attendanceSync);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-考勤同步表管理-查询考勤同步表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出考勤同步表信息", notes = "导出考勤同步表信息")
    @GetMapping(value = "/export")
    public CyResult outAttendanceSync(HttpServletResponse response,AttendanceSync attendanceSync,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="考勤同步表信息.xls";break;
            case "1": excelName="考勤同步表信息.xlsx";break;
            case "2": excelName="考勤同步表信息.csv";break;
        }
        List<AttendanceSync> attendanceSyncList = attendanceSyncServiceImpl.export(attendanceSync);
        CyEpExcelUtil.exportExcel(attendanceSyncList, "考勤同步表信息", "考勤同步表信息", AttendanceSync.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            attendanceSyncList);
    }
}
