package org.rcisoft.sys.attendance.controller;



import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.entity.ScheduledResult;
import org.rcisoft.sys.attendance.entity.ScheduledTime;
import org.rcisoft.sys.attendance.service.AttendanceSyncService;
import org.rcisoft.sys.attendance.service.ScheduledResultService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.text.ParseException;
import java.util.List;

/**
* Created by cy on 2022年11月28日 下午4:57:21.
*/
@RestController
@RequestMapping("/scheduledresult")
public class ScheduledResultController extends CyPaginationController<ScheduledResult> {


    @Autowired
    private ScheduledResultService scheduledResultService;


    @CyOpeLogAnno(title = "system-考勤同步表管理-查询考勤同步表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询考勤同步表集合", notes="分页查询考勤同步表集合")
    @GetMapping(value = "/queryScheduledResultByPagination")
    public CyGridModel listByPagination(ScheduledResult scheduledResult) {
        scheduledResultService.findAllByPagination(getPaginationUtility(), scheduledResult);
        return getGridModelResponse();
    }


    @CyOpeLogAnno(title = "system-考勤同步表管理-查询考勤同步表", businessType = CyLogTypeEnum.OTHER)
    @ApiOperation(value="执行考勤同步", notes="执行考勤同步")
    @GetMapping(value = "/implement")
    public CyResult implement(ScheduledResult scheduledResult) throws ParseException {
        CyPersistModel data = scheduledResultService.implement(scheduledResult);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                scheduledResult);
    }

    @CyOpeLogAnno(title = "system-同步通行数据管理-同步通行数据", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询同步通行集合", notes="分页查询同步通行集合")
    @GetMapping(value = "/queryScheduledTimeByPagination")
    public CyGridModel queryScheduledTimeByPagination(ScheduledTime scheduledTime) {
        scheduledResultService.queryScheduledTimeByPagination(getPaginationUtility(), scheduledTime);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-同步通行数据管理-手动同步中控通行", businessType = CyLogTypeEnum.OTHER)
    @ApiOperation(value="手动同步中控通行", notes="手动同步中控通行")
    @GetMapping(value = "/schedule")
    public CyResult schedule(ScheduledTime scheduledTime) {
        CyPersistModel data = scheduledResultService.schedule(scheduledTime);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                scheduledTime);
    }
}
