package org.rcisoft.sys.attendance.schedule;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.integration.jieLink.service.impl.ZKTecoServiceImpl;
import org.rcisoft.sys.attendance.dao.AttendanceSyncRepository;
import org.rcisoft.sys.attendance.dao.ScheduledResultRepository;
import org.rcisoft.sys.attendance.entity.Attendance;
import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.entity.ScheduledResult;
import org.rcisoft.sys.attendance.entity.ScheduledTime;
import org.rcisoft.sys.attendance.service.AttendanceService;
import org.rcisoft.sys.attendance.util.TimeUtils;
import org.rcisoft.sys.messageinstance.entity.MessageInstance;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.visit.dao.VisitInfomationRepository;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitInfomationService;
import org.rcisoft.tencent.service.impl.CyWxMiniServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static org.rcisoft.sys.attendance.service.impl.AttendanceServiceImpl.dateToWeek;

@Component
@ConditionalOnProperty(prefix = "syncSchedule", name = "config", havingValue = "true")
@Slf4j
public class AttendanceSyncSchedule {
    @Autowired
    private AttendanceService attendanceService;
    @Autowired
    private SysUserRepositorys sysUserRepositorys;

    @Autowired
    private AttendanceSyncRepository attendanceSyncRepository;

    @Autowired
    private ScheduledResultRepository scheduledResultRepository;

    @Autowired
    private VisitInfomationRepository visitInfomationRepository;

    @Autowired
    private VisitInfomationService visitInfomationServiceImpl;

    @Value("${wx.resultTemplateId}")
    private String resultTemplateId;

    @Value("${wx.arriveTemplateId}")
    private String arriveTemplateId;

    @Autowired
    private CyWxMiniServiceImpl cyWxMiniServiceImpl;

    @Autowired
    private ZKTecoServiceImpl zkTecoService;

    @Value("${message.config}")
    private boolean message;

    //    每天凌晨1点执行一次
    @Scheduled(cron = "0 0 5/7 * * ?")
//    @Scheduled(cron = "0 0/2 * * * ?")
    public void execute() {
        Date update = TimeUtils.getUpdate();
        QueryWrapper<ScheduledResult> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", "考勤同步");
        queryWrapper.eq("kq_date", TimeUtils.getDateNotTime(update));
        queryWrapper.eq("flag", "1");
        List<ScheduledResult> list = scheduledResultRepository.selectList(queryWrapper);
        if (list.isEmpty()) {
            queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("kq_date", TimeUtils.getDateNotTime(update));
            List<ScheduledResult> list1 = scheduledResultRepository.selectList(queryWrapper);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            try {
                Attendance attendance = new Attendance();
                // 查询昨日所有考勤
                List<Attendance> all = attendanceService.findAll(attendance);
                AttendanceSync attendanceSync = new AttendanceSync();
                for (int i = 0; i < all.size(); i++) {
                    //通过id查询用户表
                    SysUser sysUser = sysUserRepositorys.querySysUsersById(all.get(i).getUserId());

                    if (sysUser != null) {
                        attendanceSync = new AttendanceSync();
                        attendanceSync.setUserId(sysUser.getBusinessId());
                        attendanceSync.setName(all.get(i).getPeopleName());
                        attendanceSync.setPhone(sysUser.getPhone());
                        attendanceSync.setDeptId(sysUser.getDeptId());
                        attendanceSync.setDeptLabel(all.get(i).getDeptName());
                        attendanceSync.setPosition(sysUser.getPosition());
                        attendanceSync.setPositionLabel(all.get(i).getPositionName());
                        attendanceSync.setWorkHours(all.get(i).getWorkShift());
                        attendanceSync.setOffHours(all.get(i).getClosingTime());
                        attendanceSync.setWorkDeviceName(all.get(i).getWorkDeviceName());
                        attendanceSync.setOffDeviceName(all.get(i).getOfficeDeviceName());
                        attendanceSync.setDateOfAttendance(all.get(i).getKqDate());
                        String kqdate=attendanceSync.getDateOfAttendance();
                        String weekdays=dateToWeek(kqdate);
                        attendanceSync.setWeekday(weekdays);
                        attendanceSync.setTimeOnJob(all.get(i).getDuration());
                        attendanceSync.setNltBz(sysUser.getNltBz());
                        attendanceSync.setNltBzLabel(all.get(i).getNltBz());
                        attendanceSync.setFlag(all.get(i).getKqFlag());
                        attendanceSync.setCreateDate(new Date());
                        attendanceSync.setPassageTime(all.get(i).getCrossTime());
                        if (attendanceSync.getFlag().equals("正常")) {
                            attendanceSync.setFlag("1");
                        } else if (attendanceSync.getFlag().equals("异常")) {
                            attendanceSync.setFlag("0");
                        }
                        // 判断考勤是否已经存在，已经存在不会添加
                        QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                        wrapper.eq("date_of_attendance", attendanceSync.getDateOfAttendance());
                        wrapper.eq("user_id", attendanceSync.getUserId());
                        List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                        if (attendanceSyncList.size() == 0)
                            attendanceSyncRepository.addAttendanceSync(attendanceSync);
                    }
                }
                SysUser user = new SysUser();
                user.setFlag("1");
                user.setAttendanceShow("0");
                //查询所有在职，显示考勤的用户
                List<SysUser> users = sysUserRepositorys.querySysUsers(user);
                List<Integer> userIds = all.stream().map(Attendance::getUserId).collect(Collectors.toList());
                users = users.stream().filter(u -> !userIds.contains(u.getBusinessId())).collect(Collectors.toList());
                for (SysUser sysUser : users) {
                    attendanceSync = new AttendanceSync();
                    attendanceSync.setUserId(sysUser.getBusinessId());
                    attendanceSync.setName(sysUser.getName());
                    attendanceSync.setPhone(sysUser.getPhone());
                    attendanceSync.setDeptId(sysUser.getDeptId());
                    attendanceSync.setDeptLabel(sysUser.getDeptName());
                    attendanceSync.setPosition(sysUser.getPosition());
                    attendanceSync.setPositionLabel(sysUser.getPositionLabel());
                    attendanceSync.setWorkHours(null);
                    attendanceSync.setOffHours(null);
                    attendanceSync.setWorkDeviceName(null);
                    attendanceSync.setOffDeviceName(null);
                    attendanceSync.setDateOfAttendance(sdf.format(update));
                    String kqdate=attendanceSync.getDateOfAttendance();
                    String weekdays=dateToWeek(kqdate);
                    attendanceSync.setWeekday(weekdays);
                    attendanceSync.setTimeOnJob(BigDecimal.ZERO);
                    attendanceSync.setNltBz(sysUser.getNltBz());
                    attendanceSync.setNltBzLabel(sysUser.getNltBzLabel());
                    attendanceSync.setCreateDate(new Date());
                    attendanceSync.setPassageTime(null);
                    attendanceSync.setFlag("0");
                    // 判断考勤是否已经存在，已经存在不会添加
                    QueryWrapper<AttendanceSync> wrapper = new QueryWrapper();
                    wrapper.eq("date_of_attendance", attendanceSync.getDateOfAttendance());
                    wrapper.eq("user_id", attendanceSync.getUserId());
                    List<AttendanceSync> attendanceSyncList = attendanceSyncRepository.selectList(wrapper);
                    if (attendanceSyncList.size() == 0)
                        attendanceSyncRepository.addAttendanceSync(attendanceSync);
                }
                ScheduledResult scheduledResult = new ScheduledResult();
                scheduledResult.setType("考勤同步");
                scheduledResult.setKqDate(TimeUtils.getDateNotTime(update));
                scheduledResult.setFlag("1");
                if (list1.isEmpty())
                    scheduledResultRepository.insert(scheduledResult);
                else {
                    scheduledResult.setBusinessId(list1.get(0).getBusinessId());
                    scheduledResultRepository.updateById(scheduledResult);
                }
            } catch (Exception e) {
                ScheduledResult scheduledResult = new ScheduledResult();
                scheduledResult.setType("考勤同步");
                scheduledResult.setKqDate(TimeUtils.getDateNotTime(update));
                scheduledResult.setFlag("0");
                if (list1.isEmpty())
                    scheduledResultRepository.insert(scheduledResult);
            }
        }
    }


    @Scheduled(cron = "0 0/10 * * * ?")
//    @Scheduled(cron = "0 0/2 * * * ?")
    public void updateInfo() {
        // 获取当前时间前八个小时的时间，查询访问时间《=这个时间状态是未到访的数据批量更新为已失效
        // 判断当前时间是否已经超过访问时间8小时
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        Date date = null;
        try {
            Calendar cal = Calendar.getInstance();
            cal.setTime(format.parse(format.format(new Date())));
            cal.add(Calendar.HOUR, -8);// 24小时制
            date = cal.getTime();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        QueryWrapper<VisitInfoMation> queryWrapper = new QueryWrapper<>();
        queryWrapper.le("visit_real_time", date);
//        queryWrapper.eq("syn_status", "0");
        queryWrapper.eq("del_flag", "0");
        queryWrapper.ne("visit_status", "4").ne("visit_status", "2");//0 待访客确认 1 待审核 2已失效 3 未到访 4 已到访
        List<VisitInfoMation> list = visitInfomationRepository.selectList(queryWrapper);
        if (list.size() > 0) {
            String token = cyWxMiniServiceImpl.getAccessToken();
            MessageInstance messageInstance = new MessageInstance();
            list.forEach(l->{
                l.setUpdateType("2");
                List<VisitUser> users = visitInfomationRepository.getUserList(l.getBusinessId());
                String userNames = users.stream().map(VisitUser::getUserName).collect(Collectors.joining(","));
                String visitStatus = l.getVisitStatus();
                if ("1".equals(visitStatus)) {
                    //8.超时未审核给所有访客发送通知
                    messageInstance.setTitle("超时未审核通知");
                    messageInstance.setContent("您计划于" + format.format(l.getVisitRealTime()) + "到访国药现代制药的申请因邀请人超时未审核已失效 " +
                            "<a href=\"/pages/init/init?flag=1&id=" + l.getBusinessId() + "\" style=\"color: #3497CE;\">点击查看</a>");
//                    String userIds = users.stream().map(VisitUser::getUserId).map(String::valueOf).collect(Collectors.joining(","));
                    messageInstance.setUsers(users);
                } else if ("3".equals(visitStatus)) {
                    //7.超时未到访给邀请的员工发送通知
                    messageInstance.setTitle("访客未到访通知");
                    messageInstance.setContent("您邀请的访客"+userNames+"未在约定的访问时间到访，访客邀请已失效 " +
                            "<a href=\"/pages/init/init?flag=2&id=" + l.getBusinessId() + "\" style=\"color: #3497CE;\">点击查看</a>");
                    messageInstance.setUserIds(String.valueOf(l.getUserId()));
                }
                visitInfomationServiceImpl.check(l,messageInstance,token);
                if (message) {
                    if ("3".equals(visitStatus)) {
                        //公众号推送消息
                        SysUser user = sysUserRepositorys.getUserByBusinessId(l.getUserId());
                        Map<String, Object> map = new HashMap<>();
                        Map<String, String> keyMap = new HashMap<>();
                        keyMap.put("value", "您邀请的访客" + userNames + "未在约定的访问时间到访，访客邀请已失效");
                        keyMap.put("color", "#173177");
                        map.put("first", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("value", format.format(l.getVisitRealTime()));
                        keyMap.put("color", "#173177");
                        map.put("keyword1", keyMap);
                        String path = "/subpkg/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                        cyWxMiniServiceImpl.pushWeChatMessage(token, user.getWxOpenid(), arriveTemplateId, path, map);
                    }
                }
            });
        }
    }


    //    每天凌晨1点执行一次
    @Scheduled(cron = "0 0/10 * * * ?")
//    @Scheduled(cron = "0 0/2 * * * ?")
    public void getZKTInfo() {
        List<ScheduledTime> list = scheduledResultRepository.selectScheduledTime();
        ScheduledTime time = list.get(0);
        Date beginTime = time.getEndTime();
        Date endTime = new Date();
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(beginTime);
        cal1.add(Calendar.SECOND, 1);
        beginTime = cal1.getTime();

        cal1.setTime(endTime);
        cal1.add(Calendar.MINUTE, -10);
        endTime =  cal1.getTime();
//        System.out.println(DateUtil.format(beginTime,"yyyy-MM-dd HH:mm:ss")+"  " +DateUtil.format(endTime,"yyyy-MM-dd HH:mm:ss"));
        Date now = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        String nowTime = sdf.format(now);
        if (nowTime.equals("00:20")) { //如果查询的时间是新的一天，、则新加一条数据
            //开始时间：00:00:01
            cal1.setTime(now);
            cal1.add(Calendar.MINUTE, -20);
            cal1.add(Calendar.SECOND, 1);
            beginTime = cal1.getTime();
            //结束时间：00:10:00
            cal1.setTime(now);
            cal1.add(Calendar.MINUTE, -10);
            endTime =  cal1.getTime();
            log.error("----------同步时间段：" + DateUtil.format(beginTime,"yyyy-MM-dd HH:mm:ss")+"  " +DateUtil.format(endTime,"yyyy-MM-dd HH:mm:ss"));
            if (zkTecoService.saveDoorRecordInfo(DateUtil.format(beginTime,"yyyy-MM-dd HH:mm:ss")
                    ,DateUtil.format(endTime,"yyyy-MM-dd HH:mm:ss"))) {
                ScheduledTime scheduledTime = new ScheduledTime();
                String date = DateUtil.format(now,"yyyy-MM-dd") + " 00:00:00";
                scheduledTime.setScheduleDate(DateUtil.parseDate(date));
                scheduledTime.setBeginTime(DateUtil.parse(date,"yyyy-MM-dd HH:mm:ss"));
                scheduledTime.setEndTime(endTime);
                scheduledResultRepository.insertScheduledTime(scheduledTime);
            }
        } else {
            log.error("----------同步时间段：" + DateUtil.format(beginTime,"yyyy-MM-dd HH:mm:ss")+"  " +DateUtil.format(endTime,"yyyy-MM-dd HH:mm:ss"));
            int minute = (int) ((endTime.getTime() - beginTime.getTime()) / 1000);//求出两个时间段相差的秒数
            log.error("----------minute："+minute);
            if (minute<=600){//如果求出的秒数超出十分钟，则说明之前有报错，后续不执行
                if (zkTecoService.saveDoorRecordInfo(DateUtil.format(beginTime,"yyyy-MM-dd HH:mm:ss")
                        ,DateUtil.format(endTime,"yyyy-MM-dd HH:mm:ss"))) {
                    scheduledResultRepository.updateScheduledTime(time.getEndTime(), endTime,time.getBusinessId());
                }
            }
        }
    }
}
