package org.rcisoft.sys.attendance.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.attendance.entity.Attendance;
import org.rcisoft.sys.attendance.service.AttendanceService;
import org.rcisoft.sys.attendance.util.TimeUtils;
import org.rcisoft.sys.sysuser.dao.DoorRecordRepository;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.entity.DoorRecord;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class AttendanceServiceImpl implements AttendanceService {

    @Autowired
    DoorRecordRepository doorRecordRepository;

    @Autowired
    private SysDictDataRepositorys sysDictDataRepositorys;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Override
    public List<Attendance> findAll(Attendance attendance) throws ParseException {
        List<Attendance> attendanceList = new ArrayList<>();
        // 根据时间范围筛选所有打卡成功的记录
        // 获取前一天的时间
        // TODO: 2023/1/1  临时修改获取当天的数据
        Date update = TimeUtils.getUpdate();
        attendance.setBeginTime(TimeUtils.getDateNotTimeFour(update));
        attendance.setEndTime(TimeUtils.getDateEndDayFour(update));
        SysDictData queryDicData = new SysDictData();
        queryDicData.setDictType("officeLocation");
        queryDicData.setDelFlag("0");
        queryDicData.setFlag("1");
        List<SysDictData> officeLocationList = sysDictDataRepositorys.querySysDictDatas(queryDicData);
        if (!officeLocationList.isEmpty()){
            officeLocationList = officeLocationList.stream().sorted(Comparator.comparing(SysDictData::getDictSort)).collect(Collectors.toList());
            attendance.setOneLocation(officeLocationList.get(0).getDictValue());
            attendance.setTwoLocation(officeLocationList.get(1).getDictValue());
            queryDicData = new SysDictData();
            queryDicData.setDictType("administrative"); //行政办公楼
            queryDicData.setDelFlag("0");
            queryDicData.setFlag("1");
            List<SysDictData> administrativeList = sysDictDataRepositorys.querySysDictDatas(queryDicData);
            if (!administrativeList.isEmpty())
                attendance.setOneEquipmentIds(administrativeList.stream().map(s -> "'" + s.getDictValue() + "'").collect(Collectors.joining(",")));
            queryDicData = new SysDictData();
            queryDicData.setDictType("manufacturing"); //制造中心
            queryDicData.setDelFlag("0");
            queryDicData.setFlag("1");
            List<SysDictData> manufacturingList = sysDictDataRepositorys.querySysDictDatas(queryDicData);
            if (!manufacturingList.isEmpty())
                attendance.setTwoEquipmentIds(manufacturingList.stream().map(s -> "'" + s.getDictValue() + "'").collect(Collectors.joining(",")));
        }
        // 查询时间范围内员工的正常记录集合
        List<DoorRecord> doorRecordList = doorRecordRepository.queryAttendance(attendance);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//这里可以指定日期的格式
        //  根据用户分组(每个员工时间范围内的所有打卡记录)
//        Map<Integer, List<DoorRecord>> listMap = doorRecordList.stream()
//                .collect(Collectors.groupingBy(DoorRecord::getUserId));
        // todo 2022/12/30 根据用户分组(每个员工时间范围内的所有打卡记录)
        Map<String, List<DoorRecord>> listMap = doorRecordList.stream()
                .collect(Collectors.groupingBy(DoorRecord::getPersonId));
        for (String key : listMap.keySet()) {
            List<DoorRecord> records = listMap.get(key);
            // 某个员工某日所有通行记录
            Attendance att = new Attendance();
            att.setPeopleName(records.get(0).getName());
            att.setPhoneNumber(records.get(0).getMobile());
            att.setDeptName(records.get(0).getDeptName());
            att.setPositionName(records.get(0).getPositionName());
            att.setNltBz(records.get(0).getNltBz());
            att.setKqDate(sdf.format(update));
            att.setUserId(records.get(0).getUserId());
            att = setValue(records, att,update);
            if (att != null)
                attendanceList.add(att);
        }
        return attendanceList;
    }

    /**
     * 根据日期获取 星期
     *
     * @param datetime
     * @return
     */
    public static String dateToWeek(String datetime) {

        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd");
        String[] weekDays = {"日", "一", "二", "三", "四", "五", "六"};
        Calendar cal = Calendar.getInstance();
        Date date;
        try {
            date = f.parse(datetime);
            cal.setTime(date);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        //一周的第几天
        int w = cal.get(Calendar.DAY_OF_WEEK) - 1;
        if (w < 0)
            w = 0;
        return weekDays[w];
    }

    // 整合考勤信息
    public Attendance setValue(List<DoorRecord> records, Attendance attendance, Date update) throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
        SimpleDateFormat time = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat day = new SimpleDateFormat("yyyy-MM-dd");
        attendance.setDuration(BigDecimal.ZERO);
        String kqdate=attendance.getKqDate();
        String weekdays=dateToWeek(kqdate);
        attendance.setWeekday(weekdays);
        // 根据时间升序排序
        records = records.stream().sorted(Comparator.comparing(DoorRecord::getCrossTime)).collect(Collectors.toList());
        // 获取第一个进入时间为上班时间
        DoorRecord work = records.stream().filter(r -> r.getDeviceType().equals("1")).findFirst().orElse(null);
        //降序排序
        records = records.stream().sorted(Comparator.comparing(DoorRecord::getCrossTime).reversed()).collect(Collectors.toList());
        // 获取最后的出去数据
        DoorRecord out = records.stream().filter(r -> r.getDeviceType().equals("2")).findFirst().orElse(null);
        // 获取最后一条通行记录
        DoorRecord endRecord = records.stream().findFirst().orElse(null);
        // 存在出入数据
        if (work != null && endRecord != null && out!= null) {
            String begin = time.format(work.getCrossTime());
            if (endRecord.getDeviceType().equals("2")) {
                //如果最后一条是出
                attendance.setOfficeDeviceName(endRecord.getDeviceName());
                //如果跨天，后面+1
                if (sdf.format(endRecord.getCrossTime()).compareTo("04:00") < 0)
                    attendance.setClosingTime(sdf.format(endRecord.getCrossTime())+"(+1)");
                else
                    attendance.setClosingTime(sdf.format(endRecord.getCrossTime()));
            } else {
                //如果最后一条是入 算转天03:59后面+1
                attendance.setClosingTime("03:59(+1)");
                Calendar cal1 = Calendar.getInstance();
                cal1.setTime(update);
                cal1.add(Calendar.DATE,1);
                cal1.set(Calendar.HOUR_OF_DAY, 3);
                cal1.set(Calendar.MINUTE, 59);
                out.setCrossTime(cal1.getTime());
            }
            String end = time.format(endRecord.getCrossTime());
            if (begin.compareTo(end) > 0)
                return null;
            //如果跨天，后面+1
            if (sdf.format(work.getCrossTime()).compareTo("04:00") < 0)
                attendance.setWorkShift(sdf.format(work.getCrossTime())+"(+1)");
            else
                attendance.setWorkShift(sdf.format(work.getCrossTime()));
            attendance.setWorkDeviceName(work.getDeviceName());
        } else if (records.stream().filter(r -> r.getDeviceType().equals("2")).collect(Collectors.toList()).size() == records.size()) {
            // 如果全是外出数据（或只有一条外出），只有下班时间，没有上班时间，算为异常数据
            if (sdf.format(endRecord.getCrossTime()).compareTo("04:00") < 0)
                attendance.setClosingTime(sdf.format(endRecord.getCrossTime())+"(+1)");
            else
                attendance.setClosingTime(sdf.format(endRecord.getCrossTime()));
            attendance.setOfficeDeviceName(endRecord.getDeviceName());
        } else if (records.stream().filter(r -> r.getDeviceType().equals("1")).collect(Collectors.toList()).size() == records.size()) {
            // 如果全是进入数据（或只有一条进入），只有上班时间，下班时间无，算为异常数据
            if (sdf.format(work.getCrossTime()).compareTo("04:00") < 0)
                attendance.setWorkShift(sdf.format(work.getCrossTime())+"(+1)");
            else
                attendance.setWorkShift(sdf.format(work.getCrossTime()));
            attendance.setWorkDeviceName(work.getDeviceName());
        } else
            return null;

//        Date beginTime = null;
//        if (work != null)
//            beginTime = work.getCrossTime();
//        else
//            beginTime = time.parse(day.format(out.getCrossTime())+" 00:00:01");
//        Date endTime = null;
//        if (out != null)
//            endTime = out.getCrossTime();
//        else
//            endTime = time.parse(day.format(work.getCrossTime())+" 23:59:59");
//        attendance.setDuration(new BigDecimal(0));
        long sumTime = 0l;
///*        for (int i = 0; i < records.size(); i++) {
//            if (records.get(i).getDeviceType().equals("1"))
//                beginTime = records.get(i).getCrossTime();
//            if (records.get(i).getDeviceType().equals("2"))
//                endTime = records.get(i).getCrossTime();
//            if (beginTime != null && endTime != null && beginTime.compareTo(endTime) < 0) {
//                sumTime += TimeUtils.getDatePoor(beginTime, endTime);
//                beginTime = null;
//                endTime = null;
//            }
//        }*/
        // 查询所有班次
        SysDictData sysDictData = new SysDictData();
        sysDictData.setDictType("shift");
        List<SysDictData> shiftList = sysDictDataRepositorys.querySysDictDatas(sysDictData);
        SysDictData data = shiftList.stream().filter(s -> s.getDictLabel()
                .equals(attendance.getNltBz())).findFirst().orElse(null);
        attendance.setKqFlag("异常");
        // todo 2022/12/30 做data 的非空校验
        if (data != null && data.getRemarks() != null) {
//            // 班次最小时长
//            BigDecimal xz = new BigDecimal(data.getRemarks());
            String[] remarks = data.getRemarks().split("-");
            if (work != null && out != null) {
                String daytime = day.format(update);
                Date beginTime = time.parse(daytime +" "+ remarks[0] + ":00");
                Date endTime = time.parse(daytime+" "+ remarks[1] + ":00");

                Calendar cal1 = Calendar.getInstance();
                cal1.setTime(work.getCrossTime());
                cal1.set(Calendar.SECOND, 0);
                Date workTime = cal1.getTime();

                cal1.setTime(out.getCrossTime());
                cal1.set(Calendar.SECOND, 0);
                Date outTime = cal1.getTime();
                if (!beginTime.before(workTime)&& !endTime.after(outTime))
                    attendance.setKqFlag("正常");

            }
        }
        if (work != null && out != null) {
            sumTime = TimeUtils.getDatePoor(work.getCrossTime(), out.getCrossTime());
            if (sumTime > 0) {
                BigDecimal totalFee = new BigDecimal(sumTime);
                BigDecimal d100 = new BigDecimal(60 * 60 * 1000);
                BigDecimal big3 = totalFee.divide(d100, 1, RoundingMode.HALF_UP);
                attendance.setDuration(big3);
            }
        }
        return attendance;
    }

    public List<DoorRecord> findAllBy(Attendance attendance, HttpServletRequest request) {
        //从token获取用户id
        String userId = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_ID);
        if (userId != null) {
            attendance.setUserId(Integer.valueOf(userId));
        }
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date1 = "2022-01-01 00:00:01";
        String date2 = "2023-12-31 11:59:59";
        try {
            attendance.setBeginTime(simpleDateFormat.parse(date1));
            attendance.setEndTime(simpleDateFormat.parse(date2));
        } catch (ParseException e) {
            e.printStackTrace();
        }
        attendance.setBeginTime(TimeUtils.getDateNotTime(attendance.getBeginTime()));
        attendance.setEndTime(TimeUtils.getDateEndDay(attendance.getEndTime()));
        List<DoorRecord> doorRecordList = doorRecordRepository.queryAttendance2(attendance);
        return doorRecordList;
    }

    //   查找通行记录
    public DoorRecord findOneRecord(HttpServletRequest request) {
        Attendance attendance = new Attendance();
        //从token获取用户id
        String userId = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_ID);
        if (userId != null) {
            attendance.setUserId(Integer.valueOf(userId));
        }
//        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        String date1 = "2022-01-01 00:00:01";
//        String date2 = "2023-12-31 11:59:59";
//        try {
//            attendance.setBeginTime(simpleDateFormat.parse(date1));
//            attendance.setEndTime(simpleDateFormat.parse(date2));
//        } catch (ParseException e) {
//            e.printStackTrace();
//        }
        attendance.setBeginTime(TimeUtils.getDateNotTimeFour(new Date()));
        attendance.setEndTime(TimeUtils.getDateEndDayFour(new Date()));


        DoorRecord doorRecord = doorRecordRepository.queryOneAttendance(attendance);
        return doorRecord;
    }



    public static void main(String[] args) {
        String begin = "04:01";
        String end = "23:59";
        String a = "04:00";

        System.out.println(begin.compareTo(a));
    }
}
