package org.rcisoft.sys.doordeviceinfo.entity;

import cn.afterturn.easypoi.excel.annotation.Excel;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.*;

import org.rcisoft.core.entity.CyIdIncreEntity;
import org.rcisoft.core.entity.CyIdNotDataEntity;
import org.springframework.data.annotation.Transient;

import java.util.Date;
import java.util.List;

/**
 * Created with cy on 2022年12月23日 下午2:04:50.
 */
@Data
@TableName("door_device_info")
public class DoorDeviceInfo extends CyIdIncreEntity<DoorDeviceInfo> {
    //id
    @Transient
    @TableField(exist = false)
    private Integer id;
    /**
     * 设备所在地址中文（前端显示：所在位置）
     */
    @Excel(name = "所在位置",orderNum = "1", width = 30)
    private String deviceAddressLabel;

    //排序
    @Transient
    @TableField(exist = false)
    private String px;

    @Transient
    //标记异常（0 异常 1非异常）
    @TableField(exist = false)
    private String abnormal;

    @Transient
    /**
     * @desc 设备上报数据类型
     * open:开门
     * close:关门
     * warn:警告
     * up:设备上线
     * down:设备下线
     * @column data_type
     * @default
     */
    @TableField(exist = false)
    private String dataType;

    /**
     * 开始时间
     */
    @Transient
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private String beginTime;

    //结束时间
    @Transient
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private String endTime;

    /**
     * @desc 设备imei码（前端显示：门磁设备编号）
     * @column device_imei
     * @default
     */
    @Excel(name = "门磁设备编号",orderNum = "2", width = 20)
    private String deviceImei;
    /**
     * @desc 设备名称(前端显示：通道名称）
     * @column device_name
     * @default
     */

    @Excel(name = "通道名称",orderNum = "3", width = 20)
    private String deviceName;
    /**
     * @desc 设备所在地址
     * @column device_address
     * @default
     */
    private String deviceAddress;
    /**
     * @desc 设备所在区域（前端显示：所在楼层）
     * @column device_area
     * @default
     */
    @Excel(name = "所在楼层",orderNum = "4", width = 20)
    private String deviceArea;
    /**
     * @desc 设备所在点位
     * @column device_point
     * @default
     */
    @Excel(name = "所在点位",orderNum = "5", width = 20)
    private String devicePoint;


    /**
     * @desc 关联视频监控设备ID
     * @column link_video_id
     * @default
     */
    private Long linkVideoId;


    /**
     * 上报时间
     */
    @Excel(name = "设备上报时间", format ="yyyy/MM/dd HH:mm:ss",orderNum = "6", width = 20)
    @Transient
    @TableField(exist = false)
    private Date reportTime;

    /**
     * 抓拍图像
     */
    @Transient
    @TableField(exist = false)
    private String capturePic;

    /**
     * 抓拍图像
     */
    @Transient
    @TableField(exist = false)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date captureTime;

    /**
     * 设备上班数据内容 （告警原因）
     */
    @Excel(name = "告警原因",orderNum = "7", width = 20)
    @Transient
    @TableField(exist = false)
    private String dataVal;

    /**
     * 抓拍图像list
     */
    @Transient
    @TableField(exist = false)
    private List<String> capturePicList;

}

