package org.rcisoft.sys.doorvideodevice.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.sys.doorvideodevice.entity.DoorVideoDevice;
import org.rcisoft.sys.doorvideodevice.service.DoorVideoDeviceService;

import java.util.List;

/**
* Created by cy on 2023年1月3日 上午10:29:41.
*/
@RestController
@RequestMapping("/doorvideodevice")
public class DoorVideoDeviceController extends CyPaginationController<DoorVideoDevice> {


    @Autowired
    private DoorVideoDeviceService doorVideoDeviceServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:rVideoDevice:add')")
    @CyOpeLogAnno(title = "system-视频监控设备信息管理-新增视频监控设备信息", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加视频监控设备信息", notes="添加视频监控设备信息")
    @PostMapping(value = "/add")
    public CyResult add(@Valid DoorVideoDevice doorVideoDevice, BindingResult bindingResult) {
        CyPersistModel data = doorVideoDeviceServiceImpl.persist(doorVideoDevice);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        doorVideoDevice);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:rVideoDevice:delete')")
    @CyOpeLogAnno(title = "system-视频监控设备信息管理-删除视频监控设备信息", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除视频监控设备信息", notes="删除视频监控设备信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,DoorVideoDevice doorVideoDevice) {
        doorVideoDevice.setBusinessId(businessId);
        CyPersistModel data = doorVideoDeviceServiceImpl.remove(doorVideoDevice);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rVideoDevice:update')")
    @CyOpeLogAnno(title = "system-视频监控设备信息管理-修改视频监控设备信息", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改视频监控设备信息", notes="修改视频监控设备信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid DoorVideoDevice doorVideoDevice, BindingResult bindingResult) {
        doorVideoDevice.setBusinessId(businessId);
        CyPersistModel data = doorVideoDeviceServiceImpl.merge(doorVideoDevice);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        doorVideoDevice);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rVideoDevice:query')")
    @CyOpeLogAnno(title = "system-视频监控设备信息管理-查询视频监控设备信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一视频监控设备信息", notes="查询单一视频监控设备信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        doorVideoDeviceServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rVideoDevice:list')")
    @CyOpeLogAnno(title = "system-视频监控设备信息管理-查询视频监控设备信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询视频监控设备信息集合", notes="查询视频监控设备信息集合")
    @GetMapping(value = "/queryDoorVideoDevices")
    public CyResult queryDoorVideoDevices(DoorVideoDevice doorVideoDevice) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            doorVideoDeviceServiceImpl.findAll(doorVideoDevice));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:rVideoDevice:list')")
    @CyOpeLogAnno(title = "system-视频监控设备信息管理-查询视频监控设备信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询视频监控设备信息集合", notes="分页查询视频监控设备信息集合")
    @GetMapping(value = "/queryDoorVideoDeviceByPagination")
    public CyGridModel listByPagination(DoorVideoDevice doorVideoDevice) {
        doorVideoDeviceServiceImpl.findAllByPagination(getPaginationUtility(), doorVideoDevice);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-视频监控设备信息管理-查询视频监控设备信息", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出视频监控设备信息信息", notes = "导出视频监控设备信息信息")
    @GetMapping(value = "/export")
    public CyResult outDoorVideoDevice(HttpServletResponse response,DoorVideoDevice doorVideoDevice,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="视频监控设备信息信息.xls";break;
            case "1": excelName="视频监控设备信息信息.xlsx";break;
            case "2": excelName="视频监控设备信息信息.csv";break;
        }
        List<DoorVideoDevice> doorVideoDeviceList = doorVideoDeviceServiceImpl.export(doorVideoDevice);
        CyEpExcelUtil.exportExcel(doorVideoDeviceList, "视频监控设备信息信息", "视频监控设备信息信息", DoorVideoDevice.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            doorVideoDeviceList);
    }
}
