package org.rcisoft.sys.doorvideodevice.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.sys.doorvideodevice.dao.DoorVideoDeviceRepository;
import org.rcisoft.sys.doorvideodevice.entity.DoorVideoDevice;
import org.rcisoft.sys.doorvideodevice.service.DoorVideoDeviceService;
import org.rcisoft.core.service.CyBaseService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2023年1月3日 上午10:29:41.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class DoorVideoDeviceServiceImpl extends ServiceImpl<DoorVideoDeviceRepository,DoorVideoDevice>  implements DoorVideoDeviceService {


    /**
    * 保存 视频监控设备信息
    * @param doorVideoDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(DoorVideoDevice doorVideoDevice){
        //增加操作
        int line = baseMapper.insert(doorVideoDevice);
        log.debug(CyUserUtil.getAuthenUsername()+"新增了ID为"+
            doorVideoDevice.getBusinessId()+"的视频监控设备信息信息");
        return new CyPersistModel(line);
    }

    /**
    * 删除 视频监控设备信息
    * @param doorVideoDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(DoorVideoDevice doorVideoDevice){
        int line = baseMapper.realDelete(doorVideoDevice);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            doorVideoDevice.getBusinessId()+"的视频监控设备信息信息");
        return new CyPersistModel(line);
    }
    /**
    * 修改 视频监控设备信息
    * @param doorVideoDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(DoorVideoDevice doorVideoDevice){
        int line = baseMapper.updateById(doorVideoDevice);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ doorVideoDevice.getBusinessId()+"的视频监控设备信息信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 视频监控设备信息
    * @param id
    * @return
    */
    @Override
    public DoorVideoDevice findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 视频监控设备信息
    * @param doorVideoDevice
    * @return
    */
    @Override
    public IPage<DoorVideoDevice> findAllByPagination(CyPageInfo<DoorVideoDevice> paginationUtility,
        DoorVideoDevice doorVideoDevice){
        return baseMapper.queryDoorVideoDevicesPaged(paginationUtility,doorVideoDevice);

    }
    /**
    * 查询list 视频监控设备信息
    * @param doorVideoDevice
    * @return
    */
    @Override
    public List<DoorVideoDevice> findAll(DoorVideoDevice doorVideoDevice){
        return baseMapper.queryDoorVideoDevices(doorVideoDevice);

    }

    /**
    * 导出视频监控设备信息
    * @return
    */
    @Override
    public List<DoorVideoDevice> export(DoorVideoDevice doorVideoDevice) {
        List<DoorVideoDevice> doorVideoDeviceList = baseMapper.queryDoorVideoDevices(doorVideoDevice);
        return doorVideoDeviceList;
    }

    /**
     * 根据设备ID获取设备信息
     * @param deviceId
     * @return
     */
    @Override
    public DoorVideoDevice getDeviceInfoById(Long deviceId) {
        DoorVideoDevice doorVideoDevice = this.baseMapper.selectById(deviceId);
        return doorVideoDevice;
    }

    @Override
    public DoorVideoDevice getDeviceInfoByLinkId(String deviceImei, Long linkId) {
        QueryWrapper<DoorVideoDevice> queryWrapper = new QueryWrapper<>();
        if(StringUtils.isNotBlank(deviceImei)){
            queryWrapper.eq("link_door_device_imei",deviceImei);
        }
        if(linkId!=null && linkId!=0l){
            queryWrapper.eq("link_door_device_id",linkId);
        }
        List<DoorVideoDevice> doorVideoDevices = this.baseMapper.selectList(queryWrapper);
        if(doorVideoDevices!=null && !doorVideoDevices.isEmpty()){
            return doorVideoDevices.get(0);
        }else{
            return null;
        }
    }

}
