package org.rcisoft.sys.ossinfo.service.impl;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.ossinfo.dao.OssInfoRepository;
import org.rcisoft.sys.ossinfo.entity.OssInfo;
import org.rcisoft.sys.ossinfo.service.OssInfoService;
import org.rcisoft.sys.ossinfo.util.OssINfoUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.io.IOException;
import java.util.List;

import lombok.extern.slf4j.Slf4j;
import org.springframework.web.multipart.MultipartFile;

/**
 * Created by cy on 2022年11月21日 上午11:18:04.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class OssInfoServiceImpl extends ServiceImpl<OssInfoRepository, OssInfo> implements OssInfoService {
    @Autowired
    OssINfoUtil ossINfoUtil;

    @Value("${global.path.base_discovery}")
    private String baseDiscovery;
    /**
     * 保存 文件存储表
     *
     * @param ossInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(OssInfo ossInfo) {
        //增加操作
        int line = baseMapper.insert(ossInfo);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                ossInfo.getBusinessId() + "的文件存储表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 文件存储表
     *
     * @param ossInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(OssInfo ossInfo) {
        int line = baseMapper.realDelete(ossInfo);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                ossInfo.getBusinessId() + "的文件存储表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 文件存储表
     *
     * @param ossInfo
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(OssInfo ossInfo) {
        int line = baseMapper.updateById(ossInfo);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + ossInfo.getBusinessId() + "的文件存储表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 文件存储表
     *
     * @param id
     * @return
     */
    @Override
    public OssInfo findById(int id) {
        return baseMapper.selectById(id);
    }

    /**
     * 分页查询 文件存储表
     *
     * @param ossInfo
     * @return
     */
    @Override
    public IPage<OssInfo> findAllByPagination(CyPageInfo<OssInfo> paginationUtility,
                                              OssInfo ossInfo) {
        return baseMapper.queryOssInfosPaged(paginationUtility, ossInfo);

    }

    /**
     * 查询list 文件存储表
     *
     * @param ossInfo
     * @return
     */
    @Override
    public List<OssInfo> findAll(OssInfo ossInfo) {
        return baseMapper.queryOssInfos(ossInfo);

    }

    /**
     * 导出文件存储表
     *
     * @return
     */
    @Override
    public List<OssInfo> export(OssInfo ossInfo) {
        List<OssInfo> ossInfoList = baseMapper.queryOssInfos(ossInfo);
        return ossInfoList;
    }

    @Override
    public OssInfo filePath(MultipartFile file, String temp) throws IOException {
        OssInfo cyFileEntity = ossINfoUtil.filePath(file, temp);
        baseMapper.insert(cyFileEntity);
        cyFileEntity.setUrl(baseDiscovery + cyFileEntity.getUrl());
        return cyFileEntity;
    }

}
