package org.rcisoft.sys.ossinfo.util;

import cn.hutool.core.io.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.util.CyDateUtil;
import org.rcisoft.sys.ossinfo.entity.OssInfo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.UUID;

@Slf4j
@Component
public class OssINfoUtil {

    protected static final String SEPARATOR;
    /*date 格式*/
    protected static final String SAVING_PATTERN_DATE = "DATE";
    /*uuid 格式*/
    protected static final String SAVING_PATTERN_UUID = "UUID";

    static {
        SEPARATOR = File.separator;
    }

    @Value("${global.path.base_upload_location}")
    private String baseUploadLocation;

    public OssInfo filePath(MultipartFile multipartFile, String temp) throws IOException {
        OssInfo cyFileEntity = new OssInfo();
        // 原文件名
        String originalName = multipartFile.getOriginalFilename();
        // 后缀
        String suffix = originalName.indexOf(".") >= 0 ? originalName.substring(originalName.lastIndexOf(".")) : "";

        String finalSavingPath;
        String savingPatternOrFullName = SAVING_PATTERN_UUID;
        String savingAbsolutePath = "";
        if (StringUtils.isNotBlank(temp))
            savingAbsolutePath = baseUploadLocation + File.separator +
                temp + File.separator + CyDateUtil.getSimpleDate("yyyyMMdd", new Date());
        else
            savingAbsolutePath = baseUploadLocation + File.separator +
                    CyDateUtil.getSimpleDate("yyyyMMdd", new Date());
//        log.error("savingAbsolutePath"+savingAbsolutePath);
        String returnFinalFileName = "";
        label32:
        {
            String fileExtendsName = "." + getSuffix(multipartFile.getOriginalFilename());
            savingPatternOrFullName = savingPatternOrFullName.toUpperCase();
            switch (savingPatternOrFullName.hashCode()) {
                case 2090926:
                    if (savingPatternOrFullName.equals(SAVING_PATTERN_DATE)) {
                        returnFinalFileName = this.getCurrentDateTimeString() + fileExtendsName;
                        finalSavingPath = savingAbsolutePath + SEPARATOR + returnFinalFileName;
                        break label32;
                    }
                    break;
                case 2616251:
                    if (savingPatternOrFullName.equals(SAVING_PATTERN_UUID)) {
                        returnFinalFileName = this.getUUIDString() + fileExtendsName;
                        finalSavingPath = savingAbsolutePath + SEPARATOR + returnFinalFileName;
                        break label32;
                    }
                default: {

                }
            }
            finalSavingPath = savingAbsolutePath + SEPARATOR + returnFinalFileName;
        }
        String path = temp + File.separator + CyDateUtil.getSimpleDate("yyyyMMdd", new Date()) + File.separator + returnFinalFileName;
        cyFileEntity.setOriginalName(originalName);
        cyFileEntity.setOssKey(returnFinalFileName);
        cyFileEntity.setFileSize(multipartFile.getSize());
        cyFileEntity.setFileType(suffix.replaceAll("\\.", ""));
        cyFileEntity.setPath(path);
        cyFileEntity.setUrl(path);
        cyFileEntity.setMd5(getMd5(multipartFile));
//        File dir = new File(savingAbsolutePath);
//        if (!dir.exists()) {
//            dir.mkdirs();
//        }
//        if(!FileUtil.exist(savingAbsolutePath)){
//            FileUtil.mkdir(savingAbsolutePath);
//        }
//        log.error("savingAbsolutePath"+savingAbsolutePath);
//        log.error("finalSavingPath"+finalSavingPath);
//        try {

        File file = new File(finalSavingPath);
            if(!FileUtil.exist(file.getParentFile())){
                FileUtil.mkdir(file.getParentFile());
                log.error("file.getParentFile()"+file.getParentFile());
            }

//            log.error("file.getParentFile()"+file.getParentFile());
            if (!file.exists() && file.createNewFile())
                multipartFile.transferTo(file);
//        } catch (IllegalStateException var10) {
//            log.error(var10.getMessage());
//        } catch (IOException var11) {
//            log.error(var11.getMessage());
//        }

        return cyFileEntity;
    }

    public String getMd5(MultipartFile file) {
        try {
            byte[] uploadBytes = file.getBytes();
            //file->byte[],生成md5
            String md5Hex = DigestUtils.md5Hex(uploadBytes);
            //file->InputStream,生成md5
            String md5Hex1 = DigestUtils.md5Hex(file.getInputStream());
            //对字符串生成md5
            String s = DigestUtils.md5Hex("字符串");
            return md5Hex;
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return null;
    }

    public static String getSuffix(String filename) {
        String suffix = "";
        int pos = filename.lastIndexOf(46);
        if (pos > 0 && pos < filename.length() - 1) {
            suffix = filename.substring(pos + 1);
        }

        return suffix;
    }

    protected String getUUIDString() {
        return UUID.randomUUID().toString().replaceAll("-", "").toUpperCase();
    }

    protected String getCurrentDateTimeString() {
        return CyDateUtil.getTime_yyyyMMddHHmmss(new Date());
    }
}
