package org.rcisoft.sys.sysrole.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.rbac.role.entity.SysRoleRbac;
import org.rcisoft.sys.sysrole.service.SysRoleService;
import org.rcisoft.sys.wbac.role.dto.SysRoleDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;

import javax.validation.Valid;


/**
* Created by cy on 2022年11月21日 下午5:13:38.
*/
@RestController
@RequestMapping("/sysRole")
public class SysRoleController extends CyPaginationController<SysRoleRbac> {


    @Autowired
    private SysRoleService sysRoleServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:role:add')")
    @CyOpeLogAnno(title = "system-角色信息表管理-新增角色信息表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加角色信息表", notes="添加角色信息表")
    @PostMapping(value = "/add")
    public CyResult add(@RequestBody SysRoleRbac sysRole) {
        CyPersistModel data = sysRoleServiceImpl.persist(sysRole);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysRole);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:role:update')")
    @CyOpeLogAnno(title = "system-角色信息表管理-修改角色信息表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改角色信息表", notes="修改角色信息表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody SysRoleRbac sysRole) {
        CyPersistModel data = sysRoleServiceImpl.merge(sysRole);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysRole);
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:role:list')")
    @CyOpeLogAnno(
            title = "system-角色管理-查询角色",
            businessType = CyLogTypeEnum.QUERY
    )
    @ApiOperation(
            value = "分页查询角色",
            notes = "分页查询角色"
    )
    @GetMapping({"/list"})
    public CyGridModel listByPagination(SysRoleDTO sysRoleDTO) {
        sysRoleServiceImpl.findAllByPagination(getPaginationUtility(), sysRoleDTO);
        return this.getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-角色管理-修改角色", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "状态修改", notes = "状态修改")
    @PutMapping("/changeStatus")
    public CyResult changeStatus(@Valid SysRoleRbac sysRole, BindingResult bindingResult) {
        sysRole.setUpdateBy(CyUserUtil.getAuthenUsername());
        return CyResultGenUtil.builder(sysRoleServiceImpl.updateRoleStatus(sysRole),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysRole);
    }
}
