package org.rcisoft.sys.sysrole.service.impl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.result.enums.CyResAuthExcEnum;
import org.rcisoft.core.util.CyEscapeUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.sys.rbac.role.dao.SysRoleMenuRbacRepository;
import org.rcisoft.sys.rbac.role.dao.SysRoleRbacRepository;
import org.rcisoft.sys.rbac.role.dto.SysRoleMenuRbacDTO;
import org.rcisoft.sys.rbac.role.dto.SysRoleRbacDTO;
import org.rcisoft.sys.rbac.role.entity.SysRoleMenuRbac;
import org.rcisoft.sys.rbac.role.entity.SysRoleRbac;

import org.rcisoft.sys.sysrole.dao.SysRoleMapper;
import org.rcisoft.sys.sysrole.service.SysRoleService;
import org.rcisoft.sys.wbac.role.dto.SysRoleDTO;
import org.rcisoft.sys.wbac.role.entity.SysRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;

/**
 * Created by cy on 2022年11月21日 下午5:13:38.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysRoleServiceImpl extends ServiceImpl<SysRoleRbacRepository,SysRoleRbac>  implements SysRoleService {

    @Autowired
    private SysRoleMenuRbacRepository sysRoleMenuRbacRepository;
    @Autowired
    private SysRoleMapper sysRoleMapper;

    /**
    * 保存 角色信息表
    * @param sysRole
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysRoleRbac sysRole){
        SysRoleRbacDTO sysRoleDTO = new SysRoleRbacDTO();
        QueryWrapper<SysRoleRbac> sysRoleQueryWrapper = new QueryWrapper<>();
        //查重
        sysRoleQueryWrapper.eq("role_name", sysRole.getRoleName());
        List<SysRoleRbac> sysRoleList = this.baseMapper.selectList(sysRoleQueryWrapper);
        if (sysRoleList.size() > 0)
            throw new CyServiceException(4006, "新增角色'" + sysRole.getRoleName() + "'失败，角色名称已存在");
        //插入角色表
        int line = baseMapper.insert(sysRole);

        //插入角色-菜单表
        sysRoleDTO.setRoleName(sysRole.getRoleName());
        if (!CollectionUtils.isEmpty(sysRole.getMenuIds())) {
            List<Integer> menuIds = sysRole.getMenuIds();
            List<SysRoleMenuRbac> sysRoleMenus = new ArrayList<SysRoleMenuRbac>();
            SysRoleMenuRbac addRoleMenu;
            for (int menuId : menuIds) {
                addRoleMenu = new SysRoleMenuRbac();
                addRoleMenu.setRoleId(Integer.valueOf(sysRole.getBusinessId()));
                addRoleMenu.setMenuId(menuId);
                sysRoleMenus.add(addRoleMenu);
            }
            sysRoleMenuRbacRepository.insertRoleMenu(sysRoleMenus);
        }
        log.info(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                sysRole.getBusinessId() + "的角色表信息");
        return new CyPersistModel(line);
    }

    @Override
    public CyPersistModel merge(SysRoleRbac sysRole) {
        //该角色已拥有的菜单ID
        List<Integer> existsMenuIds = new ArrayList<Integer>();
        //该角色需要新增菜单ID
        List<Integer> addMenuIds = sysRole.getMenuIds();
        //该角色需要删除菜单ID
        List<Integer> deleteMenuIds = new ArrayList<Integer>();
        SysRoleMenuRbacDTO sysRoleMenuDTO = new SysRoleMenuRbacDTO();
        sysRoleMenuDTO.setRoleId(sysRole.getBusinessId());
        //查询 该角色的菜单权限信息
        List<SysRoleMenuRbac> sysRoleMenuList = sysRoleMenuRbacRepository.selectRoleMenu(sysRoleMenuDTO);
        for (SysRoleMenuRbac sysRoleMenu : sysRoleMenuList) {
            deleteMenuIds.add(sysRoleMenu.getMenuId());
            if (!CollectionUtils.isEmpty(addMenuIds))
                for (Integer menuId : addMenuIds) {
                    if (menuId.equals(sysRoleMenu.getMenuId())) {
                        existsMenuIds.add(menuId);
                    }
                }
        }
        if (!CollectionUtils.isEmpty(addMenuIds)) {
            deleteMenuIds.removeAll(addMenuIds);
            addMenuIds.removeAll(existsMenuIds);
        }
        //删除菜单
        if (null != deleteMenuIds && deleteMenuIds.size() > 0) {
            sysRoleMenuRbacRepository.deleteByRoleAndMenuIds(deleteMenuIds, sysRole.getBusinessId());
        }
        //新增菜单
        if (null != addMenuIds && addMenuIds.size() > 0) {
            List<SysRoleMenuRbac> sysRoleMenus = new ArrayList<SysRoleMenuRbac>();
            SysRoleMenuRbac addRoleMenu;
            for (int menuId : addMenuIds) {
                addRoleMenu = new SysRoleMenuRbac();
                addRoleMenu.setRoleId(sysRole.getBusinessId());
                addRoleMenu.setMenuId(menuId);
                sysRoleMenus.add(addRoleMenu);
            }
            sysRoleMenuRbacRepository.insertRoleMenu(sysRoleMenus);
        }
        //修改角色表中信息
        int line = baseMapper.updateById(sysRole);
        log.info(CyUserUtil.getAuthenUsername() + "修改了ID为" +
                sysRole.getBusinessId() + "的角色表信息");
        return new CyPersistModel(line);
    }
    @Override
    public IPage<SysRole> findAllByPagination(CyPageInfo<SysRoleRbac> paginationUtility, SysRoleDTO sysRoleDTO) {
        sysRoleDTO.setDeleted();
        sysRoleDTO.setRoleName(CyEscapeUtil.sqlEscape(sysRoleDTO.getRoleName()));
        sysRoleDTO.setRoleKey(CyEscapeUtil.sqlEscape(sysRoleDTO.getRoleKey()));
        return sysRoleMapper.querySysRolesPaged(paginationUtility, sysRoleDTO);
    }

    @Override
    public CyPersistModel updateRoleStatus(SysRoleRbac sysRole) {
        // 如果停用判断是否已存在用户
        if (baseMapper.countUserByRoleId(sysRole.getBusinessId()) > 0) {
            throw new CyServiceException(4309, "本数据已经被使用,不能停用");
        }
        int line = baseMapper.updateById(sysRole);
        return new CyPersistModel(line);
    }
}
