package org.rcisoft.sys.sysuser.controller;


/*固定导入*/

import cn.hutool.core.codec.Base64Encoder;
import com.google.code.kaptcha.Producer;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyEptSm4EnableAnno;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.constant.CySysCons;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.result.enums.CyReSysExcEnum;
import org.rcisoft.core.service.CyRedisService;
import org.rcisoft.core.util.*;
import org.rcisoft.integration.jieLink.service.IJieLinkService;
import org.rcisoft.sys.sysuser.dto.ExportUserDTO;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.FastByteArrayOutputStream;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;

import org.rcisoft.sys.sysuser.service.SysUserService;
import org.springframework.web.multipart.MultipartFile;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by cy on 2022年11月16日 下午3:19:19.
 */
@RestController
@RequestMapping("/sysUserManage")
public class SysUserController extends CyPaginationController<SysUser> {

    @Resource(
            name = "captchaProducer"
    )
    private Producer captchaProducer;
    @Resource(
            name = "captchaProducerMath"
    )
    private Producer captchaProducerMath;
    @Autowired(
            required = false
    )
    private CyRedisService cyRedisServiceImpl;
    @Value("${cy.captchaType}")
    private String captchaType;

    @Autowired
    private SysUserService sysUserServiceImpl;

    @Autowired
    private IJieLinkService jieLinkServiceImpl;

    /**
     * 捷顺开关
     */
    @Value("${jieLink.synchronization}")
    private boolean synchronization;

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:add')")
    @CyOpeLogAnno(title = "system-用户表管理-新增用户表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加用户表", notes = "添加用户表")
    @PostMapping(value = "/add")
    public CyResult add(@RequestBody SysUser sysUser, BindingResult bindingResult) {
        sysUser.setUserType("0");
        CyPersistModel data = sysUserServiceImpl.persist(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUser);
    }

    @CyOpeLogAnno(title = "system-用户表管理-查询用户是否为访客", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一用户表", notes = "查询单一用户表")
    @GetMapping("/persistIsVisitPerson")
    public CyResult persistIsVisitPerson(SysUser sysUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.persistIsVisitPerson(sysUser));
    }


    //@PreAuthorize("@cyPerm.hasPerm('sys:user:delete')")
    @CyOpeLogAnno(title = "system-用户表管理-删除用户表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除用户表", notes = "逻辑删除用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, SysUser sysUser) {
        sysUser.setBusinessId(businessId);
        CyPersistModel data = sysUserServiceImpl.removeLogical(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //逻辑删除
    //@PreAuthorize("@cyPerm.hasPerm('sys:user:delete')")
    @CyOpeLogAnno(title = "system-用户表管理-删除用户表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除用户表", notes = "删除用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}/{personId:\\w+}")
    public CyResult delete(@PathVariable int businessId, @PathVariable String personId, SysUser sysUser) {
        System.out.println("输出:" + businessId + personId);
        sysUser.setBusinessId(businessId);
        CyPersistModel data = sysUserServiceImpl.remove(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:update')")
    @CyOpeLogAnno(title = "system-用户表管理-修改用户表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改用户表", notes = "修改用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody SysUser sysUser, BindingResult bindingResult) {
        CyPersistModel data = sysUserServiceImpl.merge(sysUser);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUser);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:query')")
    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一用户表", notes = "查询单一用户表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable Integer businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findById(businessId));
    }

    //    //@PreAuthorize("@cyPerm.hasPerm('sys:user:list')")
    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询用户表集合", notes = "查询用户表集合")
    @GetMapping(value = "/querySysUsers")
    public CyResult querySysUsers(SysUser sysUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findAll(sysUser));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:user:list')")
    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询用户表集合", notes = "分页查询用户表集合")
    @GetMapping(value = "/querySysUserByPagination")
    public CyGridModel listByPagination(SysUser sysUser) {
        sysUser.setUserType("0");
//        System.out.println(sysUser);
        sysUserServiceImpl.findAllByPagination(getPaginationUtility(), sysUser);
        return getGridModelResponse();
    }

    //查询数据字典
    @GetMapping(value = "/queryData/{dictType:\\w+}")
    public CyResult queryData(SysUser sysUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findAllByData(sysUser));
    }

    //查询所有数据字典
    @GetMapping(value = "/queryDataAll/{dictType:\\w+}")
    public CyResult queryDataAll(SysUser sysUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findAllByDataAll(sysUser));
    }

    //查询部门
    @GetMapping(value = "/queryDept")
    public CyResult queryDept(SysUser sysUser) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.findAllByDept(sysUser));
    }

    @CyOpeLogAnno(title = "system-用户表管理-查询用户表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出用户表信息", notes = "导出用户表信息")
    @GetMapping(value = "/export")
    public CyResult outSysUser(HttpServletResponse response, SysUser sysUser, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "用户表信息.xls";
                break;
            case "1":
                excelName = "用户表信息.xlsx";
                break;
            case "2":
                excelName = "用户表信息.csv";
                break;
        }
        List<SysUser> sysUserList = sysUserServiceImpl.export(sysUser);
        CyEpExcelUtil.exportExcel(sysUserList, "用户表信息", "用户表信息", SysUser.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserList);
    }

    @CyOpeLogAnno(title = "system-用户管理-导入用户", businessType = CyLogTypeEnum.IMPORT)
    @ApiOperation(value = "导入用户信息", notes = "导入用户信息")
    @PostMapping("/import")
    @CyEptSm4EnableAnno
    public CyResult importSysUsers(@RequestBody MultipartFile file, ExportUserDTO exportUserDTO) {
        List<ExportUserDTO> exportUserDTOS = CyEpExcelUtil.importExcel(file, 0, 3, ExportUserDTO.class);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.importSysUsers(exportUserDTOS));
    }

    @CyOpeLogAnno(title = "system-用户管理-导出用户", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出用户信息空模板", notes = "导出用户信息空模板")
    @GetMapping(value = "/importTemplate")
    @CyEptSm4EnableAnno
    public CyResult outSysUserEmpty(HttpServletResponse response) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.exportEmptyTemplate(response));
    }

    @CyOpeLogAnno(title = "system-用户管理-导入用户", businessType = CyLogTypeEnum.IMPORT)
    @ApiOperation(value = "导入用户信息", notes = "导入用户信息")
    @PostMapping("/importExcel")
    @CyEptSm4EnableAnno
    public CyResult importSysUser(MultipartFile file, HttpServletResponse response) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                sysUserServiceImpl.importUserExcel(file, response));
    }

    /**
     * 获取 二维码接口地址
     * 配置文件需要 配置 前缀
     *
     * @param personId
     * @return
     * @deprecated 前端生成 二维码
     * 后端返回  base64
     */
    @GetMapping("/getTradeScan")
    @ResponseBody
    public CyResult getTradeScan(String personId) {
        // 根据用户id获取personId
        String codeUrl = "";
        if (synchronization)
            codeUrl = jieLinkServiceImpl.pullVisitUserQrCode(personId);
        else
            codeUrl = "sysUserServiceImpl.findById(businessId).toString()";
        String png_base64 = null;
        //返回二维码 并且构造支付包web支付对象
        ByteArrayOutputStream baOp = new ByteArrayOutputStream();
        try {
            BufferedImage bufImg = CyQrCodeUtil.qRCodeCommon(codeUrl);
            // 生成二维码QRCode图片
            ImageIO.write(bufImg, "png", baOp);
            byte[] bytes = baOp.toByteArray();//转换成字节
            /**
             * jdk 17 替代方案
             */
            png_base64 = Base64Encoder.encode(bytes).trim();
            png_base64 = png_base64.replaceAll("\n", "").replaceAll("\r", "");//删除 \r\n
        } catch (Exception e) {
            throw new CyServiceException(CyReSysExcEnum.ERROR);
        }
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                png_base64);
    }

    //图形验证码
    @GetMapping({"/captchaImage"})
    public CyResult getCode(HttpServletResponse response) throws IOException {
        CyResult cyResult = new CyResult();
        String uuid = CyIdGenUtil.uuid();
        String verifyKey = "captcha_codes:" + uuid;
        String capStr = null;
        String code = null;
        BufferedImage image = null;
        if ("math".equals(this.captchaType)) {
            String capText = this.captchaProducerMath.createText();
            capStr = capText.substring(0, capText.lastIndexOf("@"));
            code = capText.substring(capText.lastIndexOf("@") + 1);
            image = this.captchaProducerMath.createImage(capStr);
        } else if ("char".equals(this.captchaType)) {
            capStr = code = this.captchaProducer.createText();
            image = this.captchaProducer.createImage(capStr);
        }

        this.cyRedisServiceImpl.set(verifyKey, code, CySysCons.CAPTCHA_EXPIRATION);
        FastByteArrayOutputStream os = new FastByteArrayOutputStream();
        ImageIO.write(image, "jpg", os);
        Map map = new HashMap();
        map.put("captchaOnOff", true);
        map.put("uuid", uuid);
        map.put("img", CyBase64Util.encode(os.toByteArray()));
        map.put("code", code);
        cyResult.setSucessMessage("操作成功", map);
        return cyResult;
    }

    @PreAuthorize("@cyPerm.hasPerm('sys:dict:delete')")
    @CyOpeLogAnno(title = "system-字典类型管理-删除字典类型", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除字典类型", notes = "逻辑删除字典类型")
    @DeleteMapping("/deleteLogical")
    public CyResult dictionaryDel(SysDictData data) {
        return CyResultGenUtil.builder(sysUserServiceImpl.removeDictLogical(data),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                data);
    }

    //图形验证码
    @GetMapping({"/unregister"})
    public CyResult unregister(Integer userId) {
        return CyResultGenUtil.builder(sysUserServiceImpl.unregister(userId),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                userId);
    }
}
