package org.rcisoft.sys.sysuser.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.aop.CyPageUtilAsp;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.service.CyBaseService;

import org.rcisoft.sys.attendance.entity.AttendanceSync;
import org.rcisoft.sys.attendance.util.TimeUtils;
import org.rcisoft.sys.sysuser.dao.DoorRecordRepository;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.dto.VisitInfoUserDTO;
import org.rcisoft.sys.sysuser.entity.DoorRecord;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.rcisoft.sys.sysuser.service.DoorRecordService;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2022年11月24日 下午12:47:46.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class DoorRecordServiceImpl extends ServiceImpl<DoorRecordRepository, DoorRecord> implements DoorRecordService {
    @Autowired
    SysDictDataRepositorys sysDictDataRepositorys;

    /**
     * 保存 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(DoorRecord doorRecord) {
        //增加操作
        //TODO 需要重写
        int line = baseMapper.insert(doorRecord);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                doorRecord.getBusinessId() + "的通行记录信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(DoorRecord doorRecord) {
        //TODO 需要重写
        int line = baseMapper.realDelete(doorRecord);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                doorRecord.getBusinessId() + "的通行记录信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(DoorRecord doorRecord) {
        //TODO 需要重写
        doorRecord.setDeleted();
        int line = baseMapper.deleteById(doorRecord);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                doorRecord.getBusinessId() + "的通行记录信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(DoorRecord doorRecord) {
        //TODO 需要重写
        int line = baseMapper.updateDoorRecordById(doorRecord);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + doorRecord.getRecordId() + "的通行记录信息表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Override
    public DoorRecord findById(DoorRecord doorRecord) {
        //TODO 需要重写
        DoorRecord doorRecord1 = baseMapper.queryDoorRecordById(doorRecord.getRecordId(), doorRecord.getCrossTime());
        if (doorRecord1 != null) {
            //获取登记时间
            Date createDate = doorRecord1.getCrossTime();
            //截取登记日期
            SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
            doorRecord1.setDate(sf.format(createDate));
            //截取登记时间
            SimpleDateFormat sf2 = new SimpleDateFormat("HH:mm");
            doorRecord1.setTime(sf2.format(createDate));
            //获取登记方式id
            String cardType = doorRecord1.getCardType();
            //获取登记方式中文
            SysDictData sysDictData = sysDictDataRepositorys.queryDataByCardType(cardType);
            if (sysDictData != null) {
                doorRecord1.setCardTypeLabel(sysDictData.getDictLabel());
            }
        }
        return doorRecord1;
    }

    /**
     * 分页查询 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Override
    public IPage<DoorRecord> findAllByPagination(CyPageInfo<DoorRecord> paginationUtility, DoorRecord doorRecord) {
        //条件加上年份，默认查询当年的，因此代码里面需要判断是否传递有年代的逻辑
        Date nowYear = TimeUtils.getCurrentYear();
        Date nowYearEnd = TimeUtils.getCurrentYearEnd();
        List<Date> crossTimeRange = new ArrayList<>();
        crossTimeRange.add(0, nowYear);
        crossTimeRange.add(1, nowYearEnd);
        // 处理设备id
        if (!doorRecord.getDeviceId().equals("")) {
            String[] deviceIdArr = doorRecord.getDeviceId().split(",");
            String[] deviceIdArrTemp = new String[deviceIdArr.length];
            for (int i = 0; i < deviceIdArr.length; i++) {
                deviceIdArrTemp[i] = "\'" + deviceIdArr[i] + "\'";
            }
            String deviceIdArrStr = StringUtils.join(deviceIdArrTemp, ",");
            doorRecord.setDeviceId(deviceIdArrStr);
        }
        if (StringUtils.isNotEmpty(doorRecord.getBeginTime()) && StringUtils.isNotEmpty(doorRecord.getEndTime())) {
            try {
                SimpleDateFormat simpleDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                doorRecord.setStartTime(simpleDate.parse(doorRecord.getBeginTime()+" 00:00:00"));
                doorRecord.setCloseTime(simpleDate.parse(doorRecord.getEndTime()+" 23:59:59"));
            } catch (Exception e) {
                throw new CyServiceException("时间转换失败");
            }
        }
        IPage<DoorRecord> doorRecordIPage = baseMapper.queryDoorRecordsPagedWithCrossTimeRange(paginationUtility,
                doorRecord, crossTimeRange);
        for (int i = 0; i < doorRecordIPage.getRecords().size(); i++) {
            //获取登记时间
            Date crossTime = doorRecordIPage.getRecords().get(i).getCrossTime();
            //截取登记日期
            SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
            doorRecordIPage.getRecords().get(i).setDate(sf.format(crossTime));
            //截取登记时间
            SimpleDateFormat sf2 = new SimpleDateFormat("HH:mm");
            doorRecordIPage.getRecords().get(i).setTime(sf2.format(crossTime));
            //获取登记方式id
            String cardType = doorRecordIPage.getRecords().get(i).getCardType();
            //获取登记方式中文
            SysDictData sysDictData = sysDictDataRepositorys.queryDataByCardType(cardType);
            if (sysDictData != null) {
                doorRecordIPage.getRecords().get(i).setCardTypeLabel(sysDictData.getDictLabel());
            }
        }
        return doorRecordIPage;
    }

    /**
     * 查询list 通行记录信息表
     *
     * @param doorRecord
     * @return
     */
    @Override
    public List<DoorRecord> findAll(DoorRecord doorRecord) {
        //TODO 需要重写
        return baseMapper.queryDoorRecords(doorRecord);

    }

    /**
     * 导出通行记录信息表
     *
     * @return
     */
    @Override
    public List<DoorRecord> export(DoorRecord doorRecord) {
        //TODO 需要重写
        //条件加上年份，默认查询当年的，因此代码里面需要判断是否传递有年代的逻辑
        if (StringUtils.isNotBlank(doorRecord.getBeginTime()) && StringUtils.isNotBlank(doorRecord.getEndTime())) {
            try {
                SimpleDateFormat simpleDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                doorRecord.setStartTime(simpleDate.parse(doorRecord.getBeginTime()+" 00:00:00"));
                doorRecord.setCloseTime(simpleDate.parse(doorRecord.getEndTime()+" 23:59:59"));
            } catch (Exception e) {
                throw new CyServiceException("时间转换失败");
            }
        }
        Date nowYear = TimeUtils.getCurrentYear();
        Date nowYearEnd = TimeUtils.getCurrentYearEnd();
        List<Date> crossTimeRange = new ArrayList<>();
        crossTimeRange.add(0, nowYear);
        crossTimeRange.add(1, nowYearEnd);
        doorRecord.setCrossTimeRange(crossTimeRange);
        // 处理设备id
        if (!doorRecord.getDeviceId().equals("")) {
            String[] deviceIdArr = doorRecord.getDeviceId().split(",");
            String[] deviceIdArrTemp = new String[deviceIdArr.length];
            for (int i = 0; i < deviceIdArr.length; i++) {
                deviceIdArrTemp[i] = "\'" + deviceIdArr[i] + "\'";
            }
            String deviceIdArrStr = StringUtils.join(deviceIdArrTemp, ",");
            doorRecord.setDeviceId(deviceIdArrStr);
        }
        List<DoorRecord> doorRecordList = baseMapper.queryDoorRecords(doorRecord);
        for (int i = 0; i < doorRecordList.size(); i++) {
            //获取凭证类型
            String cardType = doorRecordList.get(i).getCardType();
            SysDictData sysDictData = sysDictDataRepositorys.queryDataByCardType(cardType);
            if (sysDictData != null) {
                doorRecordList.get(i).setCardType(sysDictData.getDictLabel());
            }
            //获取登记时间
            Date createDate = doorRecordList.get(i).getCrossTime();
            //截取登记日期
            SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
            doorRecordList.get(i).setDate(sf.format(createDate));
            //截取登记时间
            SimpleDateFormat sf2 = new SimpleDateFormat("HH:mm");
            doorRecordList.get(i).setTime(sf2.format(createDate));

            //获取同行人员身份
            String tenementType = doorRecordList.get(i).getTenementType();
            if (tenementType != null) {
                if (tenementType.equals("0")) {
                    doorRecordList.get(i).setTenementType("内部员工");
                } else if (tenementType.equals("1")) {
                    doorRecordList.get(i).setTenementType("外部访客");
                }
            }
            //获取登记状态
            String deviceType = doorRecordList.get(i).getDeviceType();
            if (deviceType != null) {
                if (deviceType.equals("1")) {
                    doorRecordList.get(i).setDeviceType("进入");
                } else if (deviceType.equals("2")) {
                    doorRecordList.get(i).setDeviceType("离开");
                }
            }
            //获取登记结果
            String recordType = doorRecordList.get(i).getRecordType();
            if (recordType != null) {
                if (recordType.equals("1")) {
                    doorRecordList.get(i).setRecordType("成功");
                } else {
                    doorRecordList.get(i).setRecordType("失败");
                }
            }
        }
        return doorRecordList;
    }

    @Override
    public void saveRecordData(DoorRecord data) {
        baseMapper.saveRecordData(data);
    }

    @Override
    public void saveRecordDataList(List<DoorRecord> list) {
        // 循环批量添加数据
        int toIndex = 1000;
        for (int i = 0; i < list.size(); i += 1000) {
            if (i + 1000 > list.size()) {
                toIndex = list.size() - i;
            }
            List<DoorRecord> list1 = list.subList(i, i + toIndex);
            baseMapper.saveRecordDataList(list1);
        }
    }

    /**
     * 分页查询 通行记录信息表
     * @return
     */
    @Override
    public IPage<VisitInfoUserDTO> queryVisitRecordByPagination(CyPageInfo<DoorRecord> paginationUtility, VisitInfoUserDTO visitUser) {
        //条件加上年份，默认查询当年的，因此代码里面需要判断是否传递有年代的逻辑
        Date nowYear = TimeUtils.getCurrentYear();
        Date nowYearEnd = TimeUtils.getCurrentYearEnd();
        List<Date> crossTimeRange = new ArrayList<>();
        crossTimeRange.add(0, nowYear);
        crossTimeRange.add(1, nowYearEnd);
        // 处理设备id
//        if (!doorRecord.getDeviceId().equals("")) {
//            String[] deviceIdArr = doorRecord.getDeviceId().split(",");
//            String[] deviceIdArrTemp = new String[deviceIdArr.length];
//            for (int i = 0; i < deviceIdArr.length; i++) {
//                deviceIdArrTemp[i] = "\'" + deviceIdArr[i] + "\'";
//            }
//            String deviceIdArrStr = StringUtils.join(deviceIdArrTemp, ",");
//            doorRecord.setDeviceId(deviceIdArrStr);
//        }
//        if (StringUtils.isNotEmpty(doorRecord.getBeginTime()) && StringUtils.isNotEmpty(doorRecord.getEndTime())) {
//            try {
//                SimpleDateFormat simpleDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//                doorRecord.setStartTime(simpleDate.parse(doorRecord.getBeginTime()+" 00:00:00"));
//                doorRecord.setCloseTime(simpleDate.parse(doorRecord.getEndTime()+" 23:59:59"));
//            } catch (Exception e) {
//                throw new CyServiceException("时间转换失败");
//            }
//        }
        IPage<VisitInfoUserDTO> userDTOIPage = baseMapper.queryVisitRecordsPagedWithCrossTimeRange(paginationUtility,
                visitUser, crossTimeRange);
        for (VisitInfoUserDTO record : userDTOIPage.getRecords()) {
            //获取登记时间
            Date crossTime = record.getCrossTime();
            //截取登记日期
            SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd");
            record.setDate(sf.format(crossTime));
            //截取登记时间
            SimpleDateFormat sf2 = new SimpleDateFormat("HH:mm");
            record.setInTime(sf2.format(crossTime));

        }
        return userDTOIPage;
    }

}
