package org.rcisoft.sys.sysuser.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;

import org.rcisoft.integration.jieLink.service.impl.JieLinkServiceImpl;
import org.rcisoft.integration.jieLink.service.impl.ZKTecoServiceImpl;
import org.rcisoft.sys.rbac.dept.entity.SysDeptRbac;
import org.rcisoft.sys.rbac.dept.service.SysDeptRbacService;
import org.rcisoft.sys.sysuser.dao.SysDeptRepositorys;
import org.rcisoft.sys.sysuser.dao.SysUserRepositorys;
import org.rcisoft.sys.sysuser.dto.SysDeptDTO;
import org.rcisoft.sys.sysuser.entity.SysDept;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.SysDeptService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

/**
 * Created by cy on 2022年11月18日 上午9:03:23.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class SysDeptServiceImpl extends ServiceImpl<SysDeptRepositorys, SysDept> implements SysDeptService {

    @Autowired
    SysDeptRepositorys sysDeptRepositorys;
    @Autowired
    SysUserRepositorys sysUserRepositorys;
    @Autowired
    private SysDeptRbacService sysDeptRbacService;
    @Autowired
    private JieLinkServiceImpl jieLinkService;
    @Autowired
    private ZKTecoServiceImpl zkTecoService;
    /**
     * 捷顺开关
     */
    @Value("${jieLink.synchronization}")
    private boolean synchronization;

    /**
     * 万傲开关
     */
    @Value("${zkteco.synchronization}")
    private boolean zktecoSynchronization;

    /**
     * 保存 部门表
     *
     * @param sysDept
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(SysDept sysDept) {
        //增加操作
        int line = baseMapper.insert(sysDept);
        if (synchronization) {
            if (!jieLinkService.pushAddDeptData(sysDept)) {
                try {
                    throw new CyServiceException(500, "添加部门失败，请稍后重试");
                } finally {
                    //手工回滚异常
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
            }
            if (zktecoSynchronization) {
                if (!zkTecoService.pushAddDeptData(sysDept)) {
                    try {
                        throw new CyServiceException(500, "添加部门失败，请稍后重试");
                    } finally {
                        //手工回滚异常
                        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    }
                }
            }
        }
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                sysDept.getBusinessId() + "的部门表信息");
        return new CyPersistModel(line);
    }

    /**
     * 删除 部门表
     *
     * @param sysDept
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(SysDept sysDept) {
        SysDept dept = baseMapper.selectById(sysDept.getBusinessId());
        sysDept.setDepartmentId(dept.getDepartmentId());
        sysDept.setZktCode(dept.getZktCode());
        int line = baseMapper.realDelete(sysDept);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                sysDept.getBusinessId() + "的部门表信息");
        if (synchronization) {
            if (!jieLinkService.pushDelDeptData(sysDept)) {
                try {
                    throw new CyServiceException(500, "删除部门失败，请稍后重试");
                } finally {
                    //手工回滚异常
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
            }
            if (zktecoSynchronization) {
                if (!zkTecoService.pushDelDeptData(sysDept)) {
                    try {
                        throw new CyServiceException(500, "删除部门失败，请稍后重试");
                    } finally {
                        //手工回滚异常
                        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    }
                }
            }
        }
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 部门表
     *
     * @param sysDept
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(SysDept sysDept) {
        SysDept dept = baseMapper.selectById(sysDept.getBusinessId());
        sysDept.setDepartmentId(dept.getDepartmentId());
        sysDept.setZktCode(dept.getZktCode());
        //查询部门是否有员工
        List<SysUser> sysUsers = sysUserRepositorys.querySysUsersByDId(sysDept.getBusinessId());
        if (sysUsers.size() > 0) {
            throw new CyServiceException(500, "该部门下存在员工，请先处理后再删除");
        }
        //查询是否有下级部门
        List<SysDept> sysDepts = sysDeptRepositorys.queryDeptByPId(sysDept.getBusinessId().toString());
        if (sysDepts.size() > 0) {
            throw new CyServiceException(500, "该部门存在下级部门，请先处理后再删除");
        }
        sysDept.setDeleted();
        int line = baseMapper.deleteById(sysDept);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                sysDept.getBusinessId() + "的部门表信息");
        if (synchronization) {
            if (!jieLinkService.pushDelDeptData(sysDept)) {
                try {
                    throw new CyServiceException(500, "删除部门失败，请稍后重试");
                } finally {
                    //手工回滚异常
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
            }
            if (zktecoSynchronization) {
                if (!zkTecoService.pushDelDeptData(sysDept)) {
                    try {
                        throw new CyServiceException(500, "删除部门失败，请稍后重试");
                    } finally {
                        //手工回滚异常
                        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    }
                }
            }
        }
        return new CyPersistModel(line);
    }

    /**
     * 修改 部门表
     *
     * @param sysDept
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(SysDept sysDept) {
        int line = baseMapper.updateById(sysDept);
        if (synchronization) {
            SysDept dept = baseMapper.selectById(sysDept.getBusinessId());
            sysDept.setDepartmentId(dept.getDepartmentId());
            sysDept.setZktCode(dept.getZktCode());
            if (!jieLinkService.pushModifyDeptData(sysDept)) {
                try {
                    throw new CyServiceException(500, "修改部门失败，请稍后重试");
                } finally {
                    //手工回滚异常
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                }
            }
            if (zktecoSynchronization) {
                if (!zkTecoService.pushAddDeptData(sysDept)) {
                    try {
                        throw new CyServiceException(500, "修改部门失败，请稍后重试");
                    } finally {
                        //手工回滚异常
                        TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    }
                }
            }
        }
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + sysDept.getBusinessId() + "的部门表信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 部门表
     *
     * @param businessId
     * @return
     */
    @Override
    public SysDept findById(String businessId) {
        return sysDeptRepositorys.querySysDeptById(businessId);
    }

    /**
     * 分页查询 部门表
     *
     * @param sysDept
     * @return
     */
    @Override
    public IPage<SysDept> findAllByPagination(CyPageInfo<SysDept> paginationUtility, SysDept sysDept) {
        //查询全部部门
        IPage<SysDept> sysDeptIPage = baseMapper.querySysDeptsPaged(paginationUtility, sysDept);
        for (int i = 0; i < sysDeptIPage.getRecords().size(); i++) {
            //查询上级部门id
            BigInteger parentId = sysDeptIPage.getRecords().get(i).getParentId();
            SysDept sysDept1 = sysDeptRepositorys.querySysDeptById(parentId.toString());
            if (sysDept1 != null) {
                sysDeptIPage.getRecords().get(i).setParentDepartment(sysDept1.getDeptName());
            }
        }
        return sysDeptIPage;
    }

    /**
     * 查询 部门表 用于树形结构显示
     *
     * @param sysDept
     * @return
     */
    @Override
    public List<SysDeptDTO> findATreeSysDept(SysDept sysDept) {
        //查询全部部门
        List<SysDeptDTO> sysDepts = baseMapper.queryTreeSysDepts(sysDept);

        // 查找上级部门
        for (int i = 0; i < sysDepts.size(); i++) {
            //查询上级部门id
            Long parentId = sysDepts.get(i).getParentId();
            SysDept sysDept1 = sysDeptRepositorys.querySysDeptById(parentId.toString());
            if (sysDept1 != null) {
                sysDepts.get(i).setParentDepartment(sysDept1.getDeptName());
            }
        }
        // 构建树形结构
        List <SysDeptRbac> sysDeptRbacs = new ArrayList<>(sysDepts);
        List<SysDeptDTO> sysDepts1 = sysDeptRbacService.buildDeptTree(sysDeptRbacs).stream().map(item -> (SysDeptDTO)item ).collect(Collectors.toList());
        return sysDepts1;
    }

    /**
     * 查询list 部门表
     *
     * @param sysDept
     * @return
     */
    @Override
    public List<SysDept> findAll(SysDept sysDept) {
        return baseMapper.querySysDepts(sysDept);

    }

    /**
     * 导出部门表
     *
     * @return
     */
    @Override
    public List<SysDept> export(SysDept sysDept) {
        List<SysDept> sysDeptList = baseMapper.querySysDepts(sysDept);
        return sysDeptList;
    }

}
