package org.rcisoft.sys.sysuser.util;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.annotation.Excel;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.DVConstraint;
import org.apache.poi.hssf.usermodel.HSSFDataValidation;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Name;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.sys.sysuser.dto.ExcelDTO;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.stream.Collectors;

/**
 * @author wangshujun
 * date 2020-07-22
 * @return
 **/
@Slf4j
public class CyEpExcelUtil {


    public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, boolean isCreateHeader, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams(title, sheetName);
        exportParams.setCreateHeadRows(isCreateHeader);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }
        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams();
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }
        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, String sheetName) {
        ExportParams exportParams = new ExportParams();
        exportParams.setSheetName(sheetName);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }
        defaultExport(list, pojoClass, fileName, response, exportParams);
    }

    public static void exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, HttpServletResponse response) {
        ExportParams exportParams = new ExportParams(title, sheetName);
        String[] format = fileName.split("\\.");
        if (format[1].equals("xlsx")) {
            exportParams.setType(ExcelType.XSSF);
        }
        defaultExport(list, pojoClass, fileName, response, exportParams);

    }

    public static void exportExcel(List<Map<String, Object>> list, String fileName, HttpServletResponse response) {
        defaultExport(list, fileName, response);
    }

    private static void defaultExport(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, ExportParams exportParams) {
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }
    }

    private static void defaultExport(List<Map<String, Object>> list, String fileName, HttpServletResponse response) {
        Workbook workbook = ExcelExportUtil.exportExcel(list, ExcelType.HSSF);
        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }
    }

    private static void downLoadExcel(String fileName, HttpServletResponse response, Workbook workbook) {
        OutputStream outputStream = null;
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            outputStream = response.getOutputStream();
            workbook.write(outputStream);
        } catch (IOException e) {
            throw new CyServiceException(e.getMessage());
        } finally {
            try {
                if (outputStream != null) {
                    outputStream.close();
                }
            } catch (IOException e) {
                log.info("context:" + e);
            }
        }
    }

    public static <T> List<T> importExcel(String filePath, Integer titleRows, Integer headerRows, Class<T> pojoClass) {
        if (StringUtils.isBlank(filePath)) {
            return null;
        }
        ImportParams params = new ImportParams();
        params.setTitleRows(titleRows);
        params.setHeadRows(headerRows);
        List<T> list = null;
        try {
            list = ExcelImportUtil.importExcel(new File(filePath), pojoClass, params);
        } catch (NoSuchElementException e) {
            throw new CyServiceException("模板不能为空");
        } catch (Exception e) {
            log.info("context:" + e);
            throw new CyServiceException(e.getMessage());
        }
        return list;
    }

    public static <T> List<T> importExcel(MultipartFile file, Integer titleRows, Integer headerRows, Class<T> pojoClass) {
        if (file == null) {
            return null;
        }
        ImportParams params = new ImportParams();
        params.setTitleRows(titleRows);
        params.setHeadRows(headerRows);
        List<T> list = null;
        try {
            list = ExcelImportUtil.importExcel(file.getInputStream(), pojoClass, params);
        } catch (Exception e) {
            throw new CyServiceException(502, "模板格式错误");
        }
        return list;
    }


    /**
     * 用户导入模板
     *
     * @param list
     * @param title
     * @param sheetName
     * @param pojoClass
     * @param fileName
     * @param response
     * @param excelDto
     */
    public static void exportExcelSelect(List<?> list, String title, String sheetName, Class<?> pojoClass, String fileName, HttpServletResponse response, ExcelDTO excelDto) {
        Date date = new Date();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy年MM月dd日");
        String time = dateFormat.format(date);
        defaultExportSelect(list, pojoClass, fileName, response, new ExportParams(title, "填报日期:" + time, sheetName), excelDto);
    }

    private static void defaultExportSelect(List<?> list, Class<?> pojoClass, String fileName, HttpServletResponse response, ExportParams exportParams, ExcelDTO excelDto) {
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
        workbook.createSheet("hidden");
        if (excelDto != null && null != excelDto.getSex() && excelDto.getSex().length > 0) {
            CyEpExcelUtil.selectList(workbook, 3, 510, 1, 1, excelDto.getSex(), "sex");
        }
        if (excelDto != null && null != excelDto.getDeptName() && excelDto.getDeptName().length > 0) {
            CyEpExcelUtil.selectList(workbook, 3, 510, 4, 4, excelDto.getDeptName(), "dept");
        }
        if (excelDto != null && null != excelDto.getYq() && excelDto.getYq().length > 0) {
            CyEpExcelUtil.selectList(workbook, 3, 510, 3, 3, excelDto.getYq(), "yq");
        }
        if (excelDto != null && null != excelDto.getZw() && excelDto.getZw().length > 0) {
            CyEpExcelUtil.selectList(workbook, 3, 510, 5, 5, excelDto.getZw(), "zw");
        }
        // 班次
        if (excelDto != null && null != excelDto.getShift() && excelDto.getShift().length > 0) {
            CyEpExcelUtil.selectList(workbook, 3, 510, 7, 7, excelDto.getShift(), "shift");
        }
//        if (excelDto != null && null != excelDto.getCarAttr() && excelDto.getCarAttr().length > 0) {
//            CyEpExcelUtil.selectList(workbook, 3, 510, 10, 10, excelDto.getCarAttr(), "carAttr");
//        }
        if (workbook != null) {
            downLoadExcel(fileName, response, workbook);
        }
    }

    public static String excelColIndexToStr(int columnIndex) {
        if (columnIndex <= 0) {
            return null;
        }
        String columnStr = "";
        columnIndex--;
        do {
            if (columnStr.length() > 0) {
                columnIndex--;
            }
            columnStr = ((char) (columnIndex % 26 + (int) 'A')) + columnStr;
            columnIndex = (int) ((columnIndex - columnIndex % 26) / 26);
        } while (columnIndex > 0);
        return columnStr;
    }

    /**
     * firstRow 開始行號 根据此项目，默认为2(下标0开始)
     * lastRow  根据此项目，默认为最大65535
     * firstCol 区域中第一个单元格的列号 (下标0开始)
     * lastCol 区域中最后一个单元格的列号
     * strings 下拉内容
     */
    public static void selectList(Workbook workbook, int firstRow, int lastRow, int firstCol, int lastCol,
                                  String[] strings, String cellName) {
        Sheet sheet = workbook.getSheetAt(0);
        Sheet hidden = workbook.getSheet("hidden");
        int index = workbook.getSheetIndex("hidden");
        int last = hidden.getLastRowNum();
        //创建单元格对象
        Cell cell = null;
        //遍历我们上面的数组，将数据取出来放到新sheet的单元格中
        for (int i = 0, length = strings.length; i < length; i++) {
            //取出数组中的每个元素
            String name = strings[i];
            //根据i创建相应的行对象（说明我们将会把每个元素单独放一行）
            Row row = null;
            if (hidden.getRow(i) == null)
                row = hidden.createRow(i);
            else
                row = hidden.getRow(i);
            //创建每一行中的第一个单元格
            cell = row.createCell(firstCol);
            //然后将数组中的元素赋值给这个单元格
            cell.setCellValue(name);
            last = hidden.getLastRowNum();
        }
        // 创建名称，可被其他单元格引用
        Name namedCell = workbook.createName();
        namedCell.setNameName(cellName);
        String letter = excelColIndexToStr(firstCol + 1);
        // 设置名称引用的公式
        namedCell.setRefersToFormula("hidden!$" + letter + "$1:$" + letter + "$" + strings.length);
        //  只对(x，x)单元格有效
        CellRangeAddressList cellRangeAddressList = new CellRangeAddressList(firstRow, lastRow, firstCol, lastCol);
        //  生成下拉框内容
        DVConstraint dvConstraint = DVConstraint.createFormulaListConstraint(cellName);
        HSSFDataValidation dataValidation = new HSSFDataValidation(cellRangeAddressList, dvConstraint);
        workbook.setSheetHidden(index, true);
//        HSSFDataValidation dataValidation = getDataValidationList4Col(sheet,firstRow,firstCol,lastRow,lastCol,strings,workbook);
        //  对sheet页生效
        sheet.addValidationData(dataValidation);
    }

    /**
     * 判断导入的Excel是否为用户信息模板
     *
     * @param file
     * @param o
     * @param titleRow
     * @return
     * @throws Exception
     */
    public static boolean isFileInUser(MultipartFile file, Object o, int titleRow) throws Exception {
        HSSFWorkbook workbook = new HSSFWorkbook(file.getInputStream());
        HSSFSheet hssfSheet = workbook.getSheetAt(0);
        Row row = hssfSheet.getRow(titleRow);
        int cellNum = row.getLastCellNum();
        List<String> containsUtils = new ArrayList<String>();
        for (int i = 0; i < cellNum; i++) {
            containsUtils.add(row.getCell(i).getStringCellValue());
        }
        Class clazz = o.getClass();
        Field field[] = clazz.getDeclaredFields();
        //数据表集和
        List<String> fileArray = new ArrayList<String>();
        for (Field field1 : field) {
            field1.setAccessible(true);
            if (field1.isAnnotationPresent(Excel.class)) {
                Excel excel = field1.getAnnotation(Excel.class);
                String name = excel.name();
                fileArray.add(name);
            }
        }
        return containsUtils.stream().sorted().collect(Collectors.joining()).equals(fileArray.stream().sorted().collect(Collectors.joining()));
    }
}
