package org.rcisoft.sys.sysusercard.controller;


/*????*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyGridModel;
import org.rcisoft.core.exception.CyServiceException;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;


import org.rcisoft.sys.sysusercard.entity.SysUserCard;
import org.rcisoft.sys.sysusercard.service.SysUserCardService;

import java.util.List;

/**
* Created by cy on Dec 28, 2022, 2:07:15 PM.
*/
@RestController
@RequestMapping("/sysusercard")
public class SysUserCardController extends CyPaginationController<SysUserCard> {


    @Autowired
    private SysUserCardService sysUserCardServiceImpl;

    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:add')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-新增人员凭证记录表", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value="添加人员凭证记录表", notes="添加人员凭证记录表")
    @PostMapping(value = "/add")
    public CyResult add(@Valid SysUserCard sysUserCard, BindingResult bindingResult) {
        CyPersistModel data = sysUserCardServiceImpl.persist(sysUserCard);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUserCard);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:delete')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-删除人员凭证记录表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="逻辑删除人员凭证记录表", notes="逻辑删除人员凭证记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId,SysUserCard sysUserCard) {
        sysUserCard.setBusinessId(businessId);
        CyPersistModel data = sysUserCardServiceImpl.removeLogical(sysUserCard);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:delete')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-删除人员凭证记录表", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value="删除人员凭证记录表", notes="删除人员凭证记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId,SysUserCard sysUserCard) {
        sysUserCard.setBusinessId(businessId);
        CyPersistModel data = sysUserCardServiceImpl.remove(sysUserCard);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:update')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-修改人员凭证记录表", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value="修改人员凭证记录表", notes="修改人员凭证记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update/{businessId:\\w+}")
    public CyResult update(@PathVariable int businessId, @Valid SysUserCard sysUserCard, BindingResult bindingResult) {
        sysUserCard.setBusinessId(businessId);
        CyPersistModel data = sysUserCardServiceImpl.merge(sysUserCard);
        return CyResultGenUtil.builder(data,
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUserCard);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:query')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-查询人员凭证记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询单一人员凭证记录表", notes="查询单一人员凭证记录表")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
        sysUserCardServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:list')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-查询人员凭证记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="查询人员凭证记录表集合", notes="查询人员凭证记录表集合")
    @GetMapping(value = "/querySysUserCards")
    public CyResult querySysUserCards(SysUserCard sysUserCard) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysUserCardServiceImpl.findAll(sysUserCard));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:userCard:list')")
    @CyOpeLogAnno(title = "system-人员凭证记录表管理-查询人员凭证记录表", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value="分页查询人员凭证记录表集合", notes="分页查询人员凭证记录表集合")
    @GetMapping(value = "/querySysUserCardByPagination")
    public CyGridModel listByPagination(SysUserCard sysUserCard) {
        sysUserCardServiceImpl.findAllByPagination(getPaginationUtility(), sysUserCard);
        return getGridModelResponse();
    }

    @CyOpeLogAnno(title = "system-人员凭证记录表管理-查询人员凭证记录表", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出人员凭证记录表信息", notes = "导出人员凭证记录表信息")
    @GetMapping(value = "/export")
    public CyResult outSysUserCard(HttpServletResponse response,SysUserCard sysUserCard,@PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName="";
        switch(excelId){
            case "0": excelName="人员凭证记录表信息.xls";break;
            case "1": excelName="人员凭证记录表信息.xlsx";break;
            case "2": excelName="人员凭证记录表信息.csv";break;
        }
        List<SysUserCard> sysUserCardList = sysUserCardServiceImpl.export(sysUserCard);
        CyEpExcelUtil.exportExcel(sysUserCardList, "人员凭证记录表信息", "人员凭证记录表信息", SysUserCard.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
            CyMessCons.MESSAGE_ALERT_SUCCESS,
            CyMessCons.MESSAGE_ALERT_ERROR,
            sysUserCardList);
    }
}
