package org.rcisoft.sys.visit.controller;


/*固定导入*/

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.core.anno.CyEptSm4EnableAnno;
import org.rcisoft.core.anno.CyOpeLogAnno;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.operlog.enums.CyLogTypeEnum;
import org.rcisoft.core.util.CyEpExcelUtil;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.messageinstance.entity.MessageInstance;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitInfomationService;
import org.rcisoft.tencent.service.impl.CyWxMiniServiceImpl;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import org.rcisoft.core.result.CyResult;
import org.rcisoft.core.util.CyResultGenUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.constant.CyMessCons;
import org.rcisoft.core.controller.CyPaginationController;
import org.rcisoft.core.model.CyGridModel;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by cy on 2022年11月16日 下午2:40:21.
 */
@RestController
@RequestMapping("/visitInfoMation")
public class VisitInfomationController extends CyPaginationController<VisitInfoMation> {


    @Autowired
    private VisitInfomationService visitInfomationServiceImpl;

    //添加邀访信息
    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Value("${wx.resultTemplateId}")
    private String resultTemplateId;

    @Autowired
    private CyWxMiniServiceImpl cyWxMiniServiceImpl;

    @Value("${message.config}")
    private boolean message;

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:add')")
    @CyOpeLogAnno(title = "system-邀访信息管理-新增邀访信息", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "添加邀访信息", notes = "添加邀访信息")
    @PostMapping(value = "/add")
    public CyResult add(@RequestBody VisitInfoMation visitInfomation,HttpServletRequest request) {
        CyPersistModel data = visitInfomationServiceImpl.persist(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:add')")
    @CyOpeLogAnno(title = "system-wx邀访信息管理-新增邀访信息", businessType = CyLogTypeEnum.INSERT)
    @ApiOperation(value = "微信小程序添加邀访信息", notes = "添加邀访信息")
    @PostMapping(value = "/wx/add")
    public CyResult wxAdd(@RequestBody VisitInfoMation visitInfomation,HttpServletRequest request) {
        String tempUserId = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_ID);
        int userId = Integer.parseInt(tempUserId);
        //添加条件添加用户id
        visitInfomation.setUserId(userId);
        visitInfomation.setVisitStatus("0");
        CyPersistModel data = visitInfomationServiceImpl.persistWX(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:delete')")
    @CyOpeLogAnno(title = "system-邀访信息管理-删除邀访信息", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "逻辑删除邀访信息", notes = "逻辑删除邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/deleteLogical/{businessId:\\w+}")
    public CyResult deleteLogical(@PathVariable int businessId, VisitInfoMation visitInfomation) {
        visitInfomation.setBusinessId(businessId);
        CyPersistModel data = visitInfomationServiceImpl.removeLogical(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:delete')")
    @CyOpeLogAnno(title = "system-邀访信息管理-禁用邀访信息", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "禁用邀访信息", notes = "禁用邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @PutMapping("/forbidden/{businessId:\\w+}")
    public CyResult forbidden(@PathVariable int businessId, VisitInfoMation visitInfomation) {
        visitInfomation.setBusinessId(businessId);
        CyPersistModel data = visitInfomationServiceImpl.forbidden(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-删除邀访信息", businessType = CyLogTypeEnum.DELETE)
    @ApiOperation(value = "删除邀访信息", notes = "删除邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @DeleteMapping("/delete/{businessId:\\w+}")
    public CyResult delete(@PathVariable int businessId, VisitInfoMation visitInfomation) {
        visitInfomation.setBusinessId(businessId);
        CyPersistModel data = visitInfomationServiceImpl.remove(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                businessId);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:update')")
    @CyOpeLogAnno(title = "system-邀访信息管理-修改邀访信息", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改邀访信息", notes = "修改邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PutMapping("/update")
    public CyResult update(@RequestBody VisitInfoMation visitInfomation, BindingResult bindingResult) {
        visitInfomation.setBusinessId(visitInfomation.getBusinessId());
        CyPersistModel data = visitInfomationServiceImpl.merge(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:query')")
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询单一邀访信息", notes = "查询单一邀访信息")
    @ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = true, dataType = "varchar")})
    @GetMapping("/detail/{businessId:\\w+}")
    public CyResult detail(@PathVariable int businessId) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.findById(businessId));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:list')")
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询邀访信息集合", notes = "查询邀访信息集合")
    @GetMapping(value = "/queryVisitInfomations")
    public CyResult queryVisitInfomations(VisitInfoMation visitInfomation) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.findAll(visitInfomation));
    }

    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:list')")
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "分页查询邀访信息集合", notes = "分页查询邀访信息集合")
    @GetMapping(value = "/queryVisitInfomationByPagination")
    public CyGridModel listByPagination(VisitInfoMation visitInfomation,HttpServletRequest request) {
        visitInfomationServiceImpl.findAllByPagination(getPaginationUtility(), visitInfomation);
        return getGridModelResponse();
    }
    //@PreAuthorize("@cyPerm.hasPerm('sys:itInfomation:list')")
    @CyOpeLogAnno(title = "system-wx小程序邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "wx小程序分页查询邀访信息集合", notes = "分页查询邀访信息集合")
    @GetMapping(value = "/queryVisitInfomationWithUserIdByPagination")
    public CyGridModel listWithUserIdByPagination(VisitInfoMation visitInfomation, HttpServletRequest request) {
        visitInfomationServiceImpl.findAllWithUserIdByPagination(getPaginationUtility(), visitInfomation ,request);
        return getGridModelResponse();
    }
    @CyOpeLogAnno(title = "system-邀访信息管理-查询邀访信息", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出邀访信息信息", notes = "导出邀访信息信息")
    @GetMapping(value = "/export")
    public CyResult outVisitInfomation(HttpServletResponse response, VisitInfoMation visitInfomation, @PathVariable @RequestParam(defaultValue = "0") String excelId) {
        String excelName = "";
        switch (excelId) {
            case "0":
                excelName = "邀访信息信息.xls";
                break;
            case "1":
                excelName = "邀访信息信息.xlsx";
                break;
            case "2":
                excelName = "邀访信息信息.csv";
                break;
        }
        List<VisitInfoMation> visitInfomationList = visitInfomationServiceImpl.export(visitInfomation);
        CyEpExcelUtil.exportExcel(visitInfomationList, "邀访信息信息", "邀访信息信息", VisitInfoMation.class, excelName, response);
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationList);
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-查询最近受邀记录", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "查询最近受邀记录", notes = "查询最近受邀记录")
    @GetMapping("/lastDetail")
    public CyResult lastDetail(HttpServletRequest request) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.lastDetail(request));
    }

    @CyOpeLogAnno(title = "system-wx邀访信息管理-查询两条最近的邀访记录", businessType = CyLogTypeEnum.QUERY)
    @ApiOperation(value = "wx查询两条最近的邀访记录", notes = "查询两条最近的邀访记录")
    @GetMapping("/getPastOrFuture")
    public CyResult getPastOrFuture(HttpServletRequest request) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.getPastOrFutureInfo(request));
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-导出邀访信息", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "导出邀访信息空模板", notes = "导出邀访信息空模板")
    @GetMapping(value = "/importTemplate")
    @CyEptSm4EnableAnno
    public CyResult outSysUserEmpty(HttpServletResponse response) {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.exportEmptyTemplate(response));
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-导入邀访信息", businessType = CyLogTypeEnum.IMPORT)
    @ApiOperation(value = "导入邀访信息", notes = "导入邀访信息")
    @PostMapping("/importExcel")
    @CyEptSm4EnableAnno
    public CyResult importSysUser(MultipartFile file,HttpServletResponse response) throws ParseException {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.importUserExcel(file, response));
    }

    @CyOpeLogAnno(title = "内勤系统-用户管理-导出用户", businessType = CyLogTypeEnum.EXPORT)
    @ApiOperation(value = "下载错误信息", notes = "下载错误信息")
    @PostMapping("/downFalseFile")
    @CyEptSm4EnableAnno
    public CyResult downFile(String fileName, HttpServletResponse response) throws Exception {
        return CyResultGenUtil.builder(new CyPersistModel(1),
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomationServiceImpl.download(fileName, response));
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-修改邀访记录状态", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "修改邀访记录状态", notes = "修改邀访记录状态")
    @PutMapping("/check")
    public CyResult check(@RequestBody VisitInfoMation visitInfomation) {
        MessageInstance messageInstance = new MessageInstance();
        VisitInfoMation inf = visitInfomationServiceImpl.findById(visitInfomation.getBusinessId());
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        //4.审核未通过给这条邀请记录的所有访客发送消息
        messageInstance.setTitle("访问审核结果");
        messageInstance.setContent("您计划于"+format.format(inf.getVisitRealTime())+"到访国药现代制药的申请审核未通过 " +
                "<a href=\"/pages/init/init?flag=1&id="+visitInfomation.getBusinessId()+"\" style=\"color: #3497CE;\">点击查看</a>");
        messageInstance.setResultUsers(inf.getUserList());
        String token = cyWxMiniServiceImpl.getAccessToken();
        CyPersistModel data = visitInfomationServiceImpl.check(visitInfomation,messageInstance,token);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }

    @CyOpeLogAnno(title = "system-邀访信息管理-员工审核通过", businessType = CyLogTypeEnum.UPDATE)
    @ApiOperation(value = "员工审核通过", notes = "员工审核通过")
    @PutMapping("/examinePass")
    public CyResult examinePass(@RequestBody VisitInfoMation visitInfomation) {
        CyPersistModel data = visitInfomationServiceImpl.examinePass(visitInfomation);
        return CyResultGenUtil.builder(data,
                CyMessCons.MESSAGE_ALERT_SUCCESS,
                CyMessCons.MESSAGE_ALERT_ERROR,
                visitInfomation);
    }
}
