package org.rcisoft.sys.visit.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.sys.visit.dao.InvitedPersonRepository;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.InvitedPersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.stream.Stream;

/**
 * 受邀信息逻辑类
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class InvitedPersonServiceImpl extends ServiceImpl<InvitedPersonRepository, VisitInfoMation> implements InvitedPersonService {

    @Autowired
    private InvitedPersonRepository invitedPersonRepository;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Autowired
    private VisitUserRepository visitUserRepository;

    /**
     * 获取最近访问安排
     *
     * @return
     */
    @Override
    @Transactional
    public VisitInfoMation getLastVisitInfo(HttpServletRequest request) {
        //获取当前时间
        Date date = new Date();
//        System.out.println("当前系统时间" + date);

        //获取当前用户手机号
        String phone = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_PHONE);
//        System.out.println("用户手机号" + phone);

        //如果电话为空 则返回空值
        if(StringUtils.isEmpty(phone)){
            return null;
        }
        //根据当前用户的手机号获取受邀记录id列表
        LambdaQueryWrapper<VisitUser> userLambdaQueryWrapper = new LambdaQueryWrapper<>();
        userLambdaQueryWrapper.select(VisitUser::getVisitId)
                .eq(VisitUser::getVisitPhone, phone);
        //查找获取受邀信息列表
        List<VisitUser> visitUsers = visitUserRepository.selectList(userLambdaQueryWrapper);
        //过滤id列表
        Stream<Integer> integerStream = visitUsers.stream().map(VisitUser::getVisitId);
        List<Integer> idList = integerStream.toList();
        //受邀列表为空 返回空
        if(idList.size()<=0){
            return null;
        }

        //查询条件 查询比当前时间大的第一条记录
        QueryWrapper<VisitInfoMation> infoMationQueryWrapper = new QueryWrapper<>();
        infoMationQueryWrapper.ge("visit_real_time", date)
                .orderByAsc("visit_real_time")
                .in("business_id", idList)
                .eq("visit_status",3)
                .last("limit 1");

        //查找对象
        VisitInfoMation visitInfoMation = invitedPersonRepository.selectOne(infoMationQueryWrapper);
//        System.out.println("请求返回数据" + visitInfoMation);

        return visitInfoMation;

    }


    @Override
    public CyPersistModel getVisitInfoById(int id) {
        return null;
    }

    /**
     * @param paginationUtility 分页
     * @param visitInfomation   查找信息
     * @return
     */
    @Override
    public IPage<VisitInfoMation> getVisitInfoListByPagination(CyPageInfo<VisitInfoMation> paginationUtility, VisitInfoMation visitInfomation, HttpServletRequest request) {
        //获取当前用户手机号
        String phone = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_PHONE);
        // 设置默认排序方式
        if (visitInfomation.getSort() == null || visitInfomation.getSort().equals("")){
            visitInfomation.setSort("3");
        }
        //查找对象
        IPage<VisitInfoMation> visitInfoMationIPage = invitedPersonRepository.queryVisitInfomationsPaged(paginationUtility, visitInfomation, phone);
        return visitInfoMationIPage;
    }

    @Override
    public List<VisitUser> getVisitUserByPhone(String visitPhone) {
        return invitedPersonRepository.getVisitUserByPhone(visitPhone);
    }

    @Override
    public VisitUser getVisitUserById(int visitId){
        VisitUser visitUser = invitedPersonRepository.getVisitUserById(visitId);
        System.out.println(visitUser);
        return invitedPersonRepository.getVisitUserById(visitId);
    }
}
