package org.rcisoft.sys.visit.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.util.FileUtils;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.jwt.util.CyJwtUtil;
import org.rcisoft.core.util.CyCompareToolUtils;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.integration.jieLink.service.impl.JieLinkServiceImpl;
import org.rcisoft.sys.messageinstance.entity.MessageInstance;
import org.rcisoft.sys.messageinstance.service.impl.MessageInstanceServiceImpl;
import org.rcisoft.sys.sysuser.dao.SysDeptRepositorys;
import org.rcisoft.sys.sysuser.dao.SysDictDataRepositorys;
import org.rcisoft.sys.sysuser.dto.ExcelDTO;
import org.rcisoft.sys.sysuser.entity.SysDept;
import org.rcisoft.sys.sysuser.entity.SysDictData;
import org.rcisoft.sys.sysuser.entity.SysUser;
import org.rcisoft.sys.sysuser.service.impl.SysUserServiceImpl;
import org.rcisoft.sys.sysuser.util.CheckUtil;
import org.rcisoft.sys.sysuser.util.CyEpExcelUtil;
import org.rcisoft.sys.visit.dao.VisitInfomationRepository;
import org.rcisoft.sys.visit.dao.VisitPersonRepositorys;
import org.rcisoft.sys.visit.dao.VisitUserRepository;
import org.rcisoft.sys.visit.dto.ExportVisitInfoDto;
import org.rcisoft.sys.visit.dto.ImportVistInfoDTO;
import org.rcisoft.sys.visit.dto.VisitInformationErrorDTO;
import org.rcisoft.sys.visit.entity.CyResSvcExcEnum;
import org.rcisoft.sys.visit.entity.VisitInfoMation;
import org.rcisoft.sys.visit.entity.VisitUser;
import org.rcisoft.sys.visit.service.VisitInfomationService;
import org.rcisoft.sys.visit.unit.IdCardUtil;
import org.rcisoft.tencent.service.impl.CyWxMiniServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Created by cy on 2022年11月16日 下午2:40:21.
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class VisitInfomationServiceImpl extends ServiceImpl<VisitInfomationRepository, VisitInfoMation> implements VisitInfomationService {

    @Autowired
    VisitUserRepository visitUserRepository;

    @Autowired
    SysDictDataRepositorys sysDictDataRepositorys;

    @Autowired
    VisitPersonRepositorys sysUserRepositorys;

    @Autowired
    SysDeptRepositorys sysDeptRepositorys;

    @Autowired
    VisitUserServiceImpl userService;

    @Autowired
    private CyJwtUtil cyJwtUtil;

    @Autowired
    JieLinkServiceImpl jieLinkService;

    @Autowired
    MessageInstanceServiceImpl messageInstanceService;

    @Value("${global.path.base_upload_location}")
    private String baseUploadLocation;

    @Value("${global.path.temp_location}")
    private String tempLocation;

    @Autowired
    private SysUserServiceImpl sysUserService;

    @Autowired
    private CyWxMiniServiceImpl cyWxMiniServiceImpl;

    /**
     * 捷顺开关
     */
    @Value("${jieLink.synchronization}")
    private boolean synchronization;

    @Value("${wx.resultTemplateId}")
    private String resultTemplateId;

    @Value("${wx.examineTemplateId}")
    private String examineTemplateId;

    @Value("${wx.inviteTemplateId}")
    private String inviteTemplateId;

    @Value("${message.config}")
    private boolean message;
    /**
     * 保存 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(VisitInfoMation visitInfomation) {
        //增加操作
        if (CollectionUtils.isEmpty(visitInfomation.getUserList()))
            throw new CyServiceException(500, "请至少新增一名访客");
        visitInfomation.setNormal();
        visitInfomation.setVisitStatus("3");
        String[] regions = visitInfomation.getVisitRegions();
        String region = "";
        if (regions != null) {
            for (String r : regions) {
                region = region + r + ',';
            }
            if (region.length() > 0) {
                region = region.substring(0, region.length() - 1);
                visitInfomation.setVisitRegion(region);
            }
        }
        String[] regionCodes = visitInfomation.getVisitRegionCodes();
        String regionCode = "";
        if (regionCodes != null) {
            for (String code : regionCodes) {
                regionCode = regionCode + code + ",";
            }
            if (regionCode.length() > 0) {
                regionCode = regionCode.substring(0, regionCode.length() - 1);
                visitInfomation.setVisitRegionCode(regionCode);
            }
        }
        visitInfomation.setItemId(UUID.randomUUID().toString());
        int line = baseMapper.insert(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        // 添加访客信息
        visitInfomation.getUserList().forEach(u -> {
            // 如果存在身份证号，判断身份证号的格式
            if (StringUtils.isNotEmpty(u.getIdCard()) && !IdCardUtil.isIdcard(u.getIdCard()))
                throw new CyServiceException(500, "访客" + u.getUserName() + "身份证号格式不符合规范，请检查!");
            // 判断当前访客(用户)是否已经存在
            SysUser sysUser = new SysUser();
            sysUser.setName(u.getUserName());
            sysUser.setPhone(u.getVisitPhone());
            sysUser.setUsername(u.getVisitPhone());
            // 添加身份证号
            sysUser.setIdNumber(u.getIdCard());
            sysUser.setNormal();
            List<SysUser> sysUserList = sysUserRepositorys.querySysUsersByPhone(u.getVisitPhone());
            // 不存在新增
            if (CollectionUtils.isEmpty(sysUserList)) {
                sysUserRepositorys.insert(sysUser);
                u.setPersonId(UUID.randomUUID().toString());
                u.setUserId(sysUser.getBusinessId());
            } else {
                // 存在覆盖
                if (sysUserList.get(0).getUserType().equals("0")) {
                    throw new CyServiceException(500, "员工不能作为访客!");
                }
                if (sysUserList.get(0).getUserType().equals("1")) {
                    if (sysUserList.get(0).getBlacklistFlag().toString().equals("1")) {
                        throw new CyServiceException(500, "不能邀请黑名单内的访客!");
                    }

                    boolean updateFlag = false;
                    if (!sysUserList.get(0).getName().equals(u.getUserName())) {
                        sysUserList.get(0).setName(u.getUserName());
                        updateFlag = true;
                    }
                    // 用户表里身份证为空 或 身份证号不同 进行覆盖
                    if (StringUtils.isEmpty(sysUserList.get(0).getIdNumber())|| (StringUtils.isNotEmpty(u.getIdCard()) && !sysUserList.get(0).getIdNumber().equals(u.getIdCard()))) {
                        sysUserList.get(0).setIdNumber(u.getIdCard());
                        updateFlag = true;
                    }

                    // 判断是否修改
                    if (updateFlag) {
                        sysUserRepositorys.updateSysUser(sysUserList.get(0));
                    }
                    u.setPersonId(UUID.randomUUID().toString());
                    u.setUserId(sysUserList.get(0).getBusinessId());
                }
            }
            u.setVisitId(visitInfomation.getBusinessId());
            u.setNormal();
            u.setState("3");
            visitUserRepository.insert(u);
            if ("0".equals(u.getType())){
                //给主访客发站内信：
                MessageInstance messageInstance = new MessageInstance();
                messageInstance.setTitle("访问邀请信息");
                messageInstance.setContent("您收到一条来自国药现代制药"+visitInfomation.getUserName()+"的邀请，请尽快补充访问信息，" +
                        "<a href=\"/pages/init/init?flag=0&id="+visitInfomation.getBusinessId()+"\">点击查看</a>");
                messageInstance.setRecipientId(BigInteger.valueOf(u.getBusinessId()));
                messageInstanceService.sendMessage(messageInstance);
                if (message) {
                    String token = cyWxMiniServiceImpl.getAccessToken();
                    //公众号推送消息
                    Map<String, Object> map = new HashMap<>();
                    Map<String, String> keyMap = new HashMap<>();
                    keyMap.put("value", "您收到一条来自国药现代制药" + visitInfomation.getUserName() + "的邀请，请尽快补充访问信息");
                    keyMap.put("color", "#173177");
                    map.put("first", keyMap);
                    keyMap = new HashMap<>();
                    keyMap.put("value", u.getUserName());
                    keyMap.put("color", "#173177");
                    map.put("keyword1", keyMap);
                    keyMap = new HashMap<>();
                    keyMap.put("value", u.getVisitPhone());
                    keyMap.put("color", "#173177");
                    map.put("keyword2", keyMap);
                    keyMap = new HashMap<>();
                    if (!"其他".equals(visitInfomation.getVisitReason()))
                        keyMap.put("value", visitInfomation.getVisitReason());
                    else
                        keyMap.put("value", visitInfomation.getVisitReasonOther());
                    keyMap.put("color", "#173177");
                    map.put("keyword3", keyMap);
                    keyMap = new HashMap<>();
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                    if (visitInfomation.getVisitRealTime() != null)
                        keyMap.put("value", format.format(visitInfomation.getVisitRealTime()));
                    else
                        keyMap.put("value", "");
                    keyMap.put("color", "#173177");
                    map.put("keyword4", keyMap);
                    SysUser user = sysUserRepositorys.getUserByBusinessId(u.getUserId());
                    String path = "/subpkgVisitors/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                    if (user != null)
                        cyWxMiniServiceImpl.pushWeChatMessage(token, user.getWxOpenid(), inviteTemplateId, path, map);
                }
            }
        });
        if (synchronization && jieLinkService.pushVisitorData(visitInfomation)) {
            visitInfomation.setSynStatus("1");
            baseMapper.updateById(visitInfomation);
        }
        return new CyPersistModel(line);
    }

    /**
     * 删除 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(VisitInfoMation visitInfomation) {
        int line = baseMapper.realDelete(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 禁用 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    public CyPersistModel forbidden(VisitInfoMation visitInfomation) {
        visitInfomation.setVisitStatus("2");
        // 禁用邀访信息
        int line = baseMapper.updateById(visitInfomation);
        // 撤销邀访
        //jieLinkService.cancelVisitorData(null, visitInfomation.getItemId());
        return new CyPersistModel(line);
    }

    /**
     * 逻辑删除 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(VisitInfoMation visitInfomation) {
        visitInfomation.setDeleted();
        int line = baseMapper.deleteById(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "逻辑删除了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(VisitInfoMation visitInfomation) {
        String regionCode = "";
        String region = "";
        for (String s : visitInfomation.getVisitRegionCodes()) {
            regionCode = regionCode + s + ',';
        }
        if (regionCode.length() > 0) {
            regionCode = regionCode.substring(0, regionCode.length() - 1);
            visitInfomation.setVisitRegionCode(regionCode);
        }
        for (String s : visitInfomation.getVisitRegions()) {
            region = region + s + ',';
        }
        if (regionCode.length() > 0) {
            region = region.substring(0, region.length() - 1);
            visitInfomation.setVisitRegion(region);
        }
        List<VisitUser> visitUsers = visitInfomation.getUserList();
        if (visitUsers.size() > 0) {
            userService.deleteByVisitId(visitInfomation.getBusinessId());
            for (VisitUser visitUser : visitUsers) {
                // 如果存在身份证号，判断身份证号的格式
                if (StringUtils.isNotEmpty(visitUser.getIdCard()) && !IdCardUtil.isIdcard(visitUser.getIdCard()))
                    throw new CyServiceException(500, "访客" + visitUser.getUserName() + "身份证号格式不符合规范，请检查!");
                visitUser.setVisitId(visitInfomation.getBusinessId());
                userService.persist(visitUser);
            }
        }
        int line = baseMapper.updateById(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + visitInfomation.getBusinessId() + "的邀访信息信息");
        return new CyPersistModel(line);
    }

    /**
     * 根据id查询 邀访信息
     *
     * @param id
     * @return
     */
    @Override
    public VisitInfoMation findById(int id) {
        VisitInfoMation v = baseMapper.queryVisitById(id);
        if (v.getVisitRegionCode() != null) {
            String[] regionCodes = v.getVisitRegionCode().split(",");
            v.setVisitRegionCodes(regionCodes);
        }
        if (v.getVisitRegion() != null) {
            String[] regions = v.getVisitRegion().split(",");
            v.setVisitRegions(regions);
        }
        String vuList = "";
        for (VisitUser vu : v.getUserList()) {
            vuList = vuList + vu.getUserName() + ',';
        }
        if (vuList.length() > 0) {
            vuList = vuList.substring(0, vuList.length() - 1);
        }
        v.setUserArry(vuList);
        return v;
    }

    /**
     * 分页查询 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public IPage<VisitInfoMation> findAllByPagination(CyPageInfo<VisitInfoMation> paginationUtility,
                                                      VisitInfoMation visitInfomation) {
        if (visitInfomation.getVisitReasonCode() != null && visitInfomation.getVisitReasonCode().length() > 0) {
            String[] reasonCodes = visitInfomation.getVisitReasonCode().split(",");
            for (String r : reasonCodes) {
                if (r.equals("other")) {
                    visitInfomation.setBoolExistOtherReason("YES");
                }
            }
            visitInfomation.setVisitReasonCodes(reasonCodes);
        }
        if (visitInfomation.getVisitRegionCode() != null && visitInfomation.getVisitRegionCode().length() > 0) {
            String[] regionCodes = visitInfomation.getVisitRegionCode().split(",");
            visitInfomation.setVisitRegionCodes(regionCodes);
        }
        IPage<VisitInfoMation> list = baseMapper.queryVisitInfomationsPaged(paginationUtility, visitInfomation);
        List<VisitInfoMation> mationList = list.getRecords();
        for (VisitInfoMation v : mationList) {
            if (v.getVisitRegionCode() != null) {
                String[] regionCodes = v.getVisitRegionCode().split(",");
                v.setVisitRegionCodes(regionCodes);
            }
            if (v.getVisitRegion() != null) {
                String[] regions = v.getVisitRegion().split(",");
                v.setVisitRegions(regions);
            }
            String vuList = "";
            String phoneList = "";
            for (VisitUser vu : v.getUserList()) {
                vuList = vuList + vu.getUserName() + ',';
                phoneList = phoneList + vu.getVisitPhone() + ',';
            }
            if (vuList.length() > 0) {
                vuList = vuList.substring(0, vuList.length() - 1);
                phoneList = phoneList.substring(0, phoneList.length() - 1);
            }
            v.setPhoneArry(phoneList);
            v.setUserArry(vuList);
        }
        list.setRecords(mationList);
        return list;

    }

    /**
     * 分页查询 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public IPage<VisitInfoMation> findAllWithUserIdByPagination(CyPageInfo<VisitInfoMation> paginationUtility, VisitInfoMation visitInfomation, HttpServletRequest request) {
        String tempUserId = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_ID);
        //判断是否获取到当前用户id
        //如果没有当前用户id 返回空列表
        if (StringUtils.isEmpty(tempUserId)) {
            return null;
        }
        //userId 类型转换
        int userId = Integer.parseInt(tempUserId);
        /*
            根据id进行数据查询 然后处理进行放回 //新增接口,同时前端更换接口
         */
        if (visitInfomation.getVisitReasonCode() != null && visitInfomation.getVisitReasonCode().length() > 0) {
            String[] reasonCodes = visitInfomation.getVisitReasonCode().split(",");
            for (String r : reasonCodes) {
                if (r.equals("other")) {
                    visitInfomation.setBoolExistOtherReason("YES");
                }
            }
            visitInfomation.setVisitReasonCodes(reasonCodes);
        }
        if (visitInfomation.getVisitRegionCode() != null && visitInfomation.getVisitRegionCode().length() > 0) {
            String[] regionCodes = visitInfomation.getVisitRegionCode().split(",");
            visitInfomation.setVisitRegionCodes(regionCodes);
        }
        //调用mapper层方法进行查询
        IPage<VisitInfoMation> list = baseMapper.queryVisitInfomationsByUserIdPaged(paginationUtility, visitInfomation, userId);
        List<VisitInfoMation> mationList = list.getRecords();
        for (VisitInfoMation v : mationList) {
            if (v.getVisitRegionCode() != null) {
                String[] regionCodes = v.getVisitRegionCode().split(",");
                v.setVisitRegionCodes(regionCodes);
            }
            if (v.getVisitRegion() != null) {
                String[] regions = v.getVisitRegion().split(",");
                v.setVisitRegions(regions);
            }
            String vuList = "";
            for (VisitUser vu : v.getUserList()) {
                vuList = vuList + vu.getUserName() + ',';
            }
            if (vuList.length() > 0) {
                vuList = vuList.substring(0, vuList.length() - 1);
            }
            v.setUserArry(vuList);
        }
        list.setRecords(mationList);
//        System.out.println(list);
        return list;

    }

    /**
     * 查询list 邀访信息
     *
     * @param visitInfomation
     * @return
     */
    @Override
    public List<VisitInfoMation> findAll(VisitInfoMation visitInfomation) {
        List<VisitInfoMation> visitInfomationList = baseMapper.queryVisitList(visitInfomation);
        for (VisitInfoMation v : visitInfomationList) {
            String vuList = "";
            for (VisitUser vu : v.getUserList()) {
                vuList = vuList + vu.getUserName() + ',';
            }
            vuList = vuList.substring(0, vuList.length() - 1);
            v.setUserArry(vuList);
        }
        return visitInfomationList;
    }

    /**
     * 导出邀访信息
     *
     * @return
     */
    @Override
    public List<VisitInfoMation> export(VisitInfoMation visitInfomation) {
        if (visitInfomation.getVisitReasonCode() != null && visitInfomation.getVisitReasonCode().length() > 0) {
            String[] reasonCodes = visitInfomation.getVisitReasonCode().split(",");
            for (String r : reasonCodes) {
                if (r.equals("other")) {
                    visitInfomation.setBoolExistOtherReason("YES");
                }
            }
            visitInfomation.setVisitReasonCodes(reasonCodes);
        }
        if (visitInfomation.getVisitRegionCode() != null && visitInfomation.getVisitRegionCode().length() > 0) {
            String[] regionCodes = visitInfomation.getVisitRegionCode().split(",");
            visitInfomation.setVisitRegionCodes(regionCodes);
        }
        List<VisitInfoMation> visitInfomationList = baseMapper.exportList(visitInfomation);
        for (VisitInfoMation v : visitInfomationList) {
            String vuList = "";
            String phoneList = "";
            String idCard = "";
            for (VisitUser vu : v.getUserList()) {
                if (vu.getType().equals("0"))
                    v.setMainPhone(vu.getVisitPhone());
                vuList = vuList + vu.getUserName() + ',';
                phoneList = phoneList + vu.getVisitPhone() + ',';
                if (StringUtils.isNotBlank(vu.getIdCard()))
                    idCard = idCard + vu.getIdCard() + ',';
                else
                    idCard = idCard + "空白" + ',';
            }
            if (vuList.length() > 0) {
                vuList = vuList.substring(0, vuList.length() - 1);
                phoneList = phoneList.substring(0, phoneList.length() - 1);
                idCard = idCard.substring(0, idCard.length() - 1);
            }
            v.setUserArry(vuList);
            v.setPhoneArry(phoneList);
            v.setIdCard(idCard);
            if (v.getVisitStatus() != null) {
                if (v.getVisitStatus().equals("0")) {
                    v.setVisitStatus("待访客确认");
                }
                if (v.getVisitStatus().equals("1")) {
                    v.setVisitStatus("待审核");
                }
                if (v.getVisitStatus().equals("2")) {
                    v.setVisitStatus("已失效");
                }
                if (v.getVisitStatus().equals("3")) {
                    v.setVisitStatus("未到访");
                }
                if (v.getVisitStatus().equals("4")) {
                    v.setVisitStatus("已到访");
                }
            }
        }
        return visitInfomationList;
    }

    /**
     * 获取当前用户最新受邀记录
     *
     * @return
     */
    @Override
    public VisitInfoMation lastDetail(HttpServletRequest request) {
        // 获取当前登录用户的手机号
        String phone = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_PHONE);
        if (StringUtils.isEmpty(phone))
            return null;
        // 获取当前登录用户的受邀记录
        List<VisitInfoMation> visitInfList = baseMapper.lastDetail(phone);
        if (CollectionUtils.isEmpty(visitInfList) || visitInfList.size() == 0)
            return null;
        else
            return visitInfList.get(0);
    }

    /**
     * 获取最近两条邀请记录
     *
     * @param request
     * @return
     */
    @Override
    public List<VisitInfoMation> getPastOrFutureInfo(HttpServletRequest request) {

        //获取用户id
        String tempUserId = CyUserUtil.getUserInfoStrProp(cyJwtUtil.getToken(request), CyUserUtil.USER_ID);
//        System.out.println("用户id" + tempUserId);

        //判断是否获取到当前用户id
        //如果没有当前用户id 返回空列表
        if (StringUtils.isEmpty(tempUserId)) {
            return new ArrayList<>();
        }
        //userId 类型转换
        int userId = Integer.parseInt(tempUserId);
        List<VisitInfoMation> resList = new ArrayList<>();

        //根据当前用户id查找过去的邀请记录
        List<VisitInfoMation> pastList = baseMapper.queryPast(userId);
        int pastListLength = pastList.size();

        //根据当前用户id查找未来的邀请记录
        List<VisitInfoMation> futureList = baseMapper.queryFuture(userId);
        int futureListLength = futureList.size();

        if (pastListLength > 0 && futureListLength > 0) {
            resList.add(futureList.get(0));
            resList.add(pastList.get(0));
            return resList;
        }
        if (pastListLength == 0 && futureListLength > 1) {
            resList.add(futureList.get(1));
            resList.add(futureList.get(0));
            return resList;
        }
        if (pastListLength == 0 && futureListLength == 1) {
            resList.add(futureList.get(0));
            return resList;
        }
        if (pastListLength > 1 && futureListLength == 0) {
            resList.add(pastList.get(0));
            resList.add(pastList.get(1));
            return resList;
        }
        if (pastListLength == 1 && futureListLength == 0) {
            resList.add(pastList.get(0));
            return resList;
        }
        return resList;
    }

    @Override
    public int exportEmptyTemplate(HttpServletResponse response) {
        List<ExportVisitInfoDto> imOutSysUsers = new ArrayList<>();
        ExportVisitInfoDto exportUserDTO = new ExportVisitInfoDto();
        for (int i = 0; i < 500; i++) {
            imOutSysUsers.add(exportUserDTO);
        }
        CyEpExcelUtil.exportExcelSelect(imOutSysUsers, "邀访信息", "邀访",
                ExportVisitInfoDto.class, "邀访信息模板.xls", response, null);
        return 1;
    }

    @Override
    public ImportVistInfoDTO importUserExcel(MultipartFile file, HttpServletResponse response) throws ParseException {
        List<VisitInformationErrorDTO> errorDTOArrayList = new ArrayList<>();
        ExportVisitInfoDto exportVisitInfoDto = new ExportVisitInfoDto();
        VisitInformationErrorDTO errorDTO = new VisitInformationErrorDTO();
        boolean flag = false;
        try {
            flag = CyEpExcelUtil.isFileInUser(file, exportVisitInfoDto, 2);
        } catch (Exception e) {
            throw new CyServiceException(500, "非模板文件");
        }
        if (!flag) {
            throw new CyServiceException(500, "非模板文件");
        }
        /** 从excel表中读到数据，有一部分为空值 */
        List<ExportVisitInfoDto> visitInfoDtoList = CyEpExcelUtil.importExcel(file, 2, 1, ExportVisitInfoDto.class);
        /** 校验后没有问题的数据 */
        List<ExportVisitInfoDto> addINfoList = new ArrayList<>();
        /** 遍历 从Excel表中读到的数据 抛出不合适的数据 */
        //判断整个Excel是不是为空 计算非空数据有多少条
        int information = CyCompareToolUtils.cmpareInformation(visitInfoDtoList);
        if (information == 0) {
            throw new CyServiceException(500, "Excel文件不可为空，请检查Excel文件是否填写！");
        } else if (information > 500) {
            throw new CyServiceException(500, "单次导入数据不可超过500，请检查！");
        } else {
            //去除空数据
            try {
                visitInfoDtoList = sysUserService.ThrowNull(visitInfoDtoList);
            } catch (Exception e) {
                throw new CyServiceException(CyResSvcExcEnum.ERROR_NETWORK);
            }
            //校验 将错误信息放入maps
            for (ExportVisitInfoDto visitInfoDto : visitInfoDtoList) {
                //判断 是否为空
                VisitInformationErrorDTO infoNull = CheckUtil.infoNotNull(visitInfoDto);
                if (infoNull != null)
                    errorDTOArrayList.add(infoNull);
                else
                    addINfoList.add(visitInfoDto);
            }
        }
        if (addINfoList.size() > 0) {
            for (ExportVisitInfoDto visitInfoDto : addINfoList) {
                VisitInfoMation visitInfoMation = new VisitInfoMation();
                // 根据姓名查询
                SysUser sysUserDTO = new SysUser();
                sysUserDTO.setDeleted();
                sysUserDTO.setName(visitInfoDto.getUserName());
                sysUserDTO.setPhone(visitInfoDto.getPhone());
                sysUserDTO.setUserType("0");
                // 根据姓名查询员工数据
                List<SysUser> sysUsers = sysUserRepositorys.querySysUsers(sysUserDTO);
                if (sysUsers.size() == 0) {
                    BeanUtil.copyProperties(visitInfoDto, errorDTO);
                    errorDTO.setError("不存在对应的员工，请检查邀请人姓名和手机号是否匹配。");
                    errorDTOArrayList.add(errorDTO);
                    continue;
                }
                SysUser mainUser = sysUsers.get(0);
                visitInfoMation.setUserId(mainUser.getBusinessId());
                visitInfoMation.setUserName(visitInfoDto.getUserName());
                // 设置邀访信息部门名称
                String deptId = mainUser.getDeptId();
                if (deptId != null) {
                    if (sysDeptRepositorys.queryDeptById(deptId).size() > 0) {
                        SysDept dept = sysDeptRepositorys.queryDeptById(deptId).get(0);
                        if (dept != null) {
                            // 添加部门简称
                            String deptName = dept.getDeptNameAbbreviation();
                            visitInfoMation.setDeptName(deptName);
                        }
                    }
                }
                // 设置邀访信息职位名称
                String position = mainUser.getPosition();
                if (position != null) {
                    SysDictData sysDictData = new SysDictData();
                    sysDictData.setDictType("position");
                    List<SysDictData> dictDataList = sysDictDataRepositorys.querySysDictDatas(sysDictData);
                    String postName = "";
                    for (SysDictData data : dictDataList) {
                        if (data.getDictValue().equals(position)) {
                            postName = data.getDictLabel();
                        }
                    }
                    visitInfoMation.setPostName(postName);
                }
                // 设置邀访信息访问事由
                String reason = visitInfoDto.getVisitReason();
                if (reason != null) {
                    SysDictData sysDictData = new SysDictData();
                    sysDictData.setDictType("visitReason");
                    List<SysDictData> dictDataList = sysDictDataRepositorys.querySysDictDatas(sysDictData);
                    String reasonCode = "";
                    for (SysDictData data : dictDataList) {
                        if (data.getDictLabel().equals(reason)) {
                            reasonCode = data.getDictValue();
                        }
                    }
                    if (reasonCode.equals("")) {
                        visitInfoMation.setVisitReasonCode("other");
                        visitInfoMation.setVisitReason("其他");
                        visitInfoMation.setVisitReasonOther(reason);
                    } else {
                        visitInfoMation.setVisitReason(reason);
                        visitInfoMation.setVisitReasonCode(reasonCode);
                    }
                }
                // 设置邀访信息访问区域
                if (visitInfoDto.getVisitRegion().length() > 0) {
                    String[] regions = visitInfoDto.getVisitRegion().split(",|，");

                    Set set = new HashSet();

                    for (int i = 0; i < regions.length; i++) {
                        set.add(regions[i]);
                    }
                    regions = (String[]) set.toArray(new String[set.size()]);

                    visitInfoMation.setVisitRegion(visitInfoDto.getVisitRegion());
                    String regionCode = "";
                    SysDictData queryDicData = new SysDictData();
                    queryDicData.setDictType("visitRegion");
                    queryDicData.setDelFlag("0");
                    queryDicData.setFlag("1");
                    List<SysDictData> dictDataList = sysDictDataRepositorys.querySysDictDatas(queryDicData);
                    for (String region : regions) {
                        for (SysDictData sysDictData : dictDataList) {
                            if (region.equals(sysDictData.getDictLabel())) {
                                regionCode = regionCode + sysDictData.getDictValue() + ",";
                            }
                        }
                    }
                    if (regionCode.length() > 0) {
                        regionCode = regionCode.substring(0, regionCode.length() - 1);
                        visitInfoMation.setVisitRegionCode(regionCode);
                    } else {
                        BeanUtil.copyProperties(visitInfoDto, errorDTO);
                        errorDTO.setError("访问区域不存在!");
                        errorDTOArrayList.add(errorDTO);
                        continue;
                    }
                }
                visitInfoMation.setVisitRegion(visitInfoDto.getVisitRegion());
                SimpleDateFormat date = new SimpleDateFormat("yyyy-MM-dd");
                visitInfoMation.setVisitDate(date.parse(visitInfoDto.getVisitDate()));
                visitInfoMation.setVisitTime(visitInfoDto.getVisitTime());
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                String time = visitInfoDto.getVisitDate() + " " + visitInfoDto.getVisitTime();
                visitInfoMation.setVisitRealTime(dateFormat.parse(time));
                List<VisitUser> userList = new ArrayList<>();
                // 判断访客姓名和手机号是否数量匹配
                List<String> visitPhone = Arrays.asList(visitInfoDto.getVisitPhone().split(",|，")).stream()
                        .filter(p -> StringUtils.isNotEmpty(p)).distinct()
                        .collect(Collectors.toList());
                List<String> idCard = null;
                if (StringUtils.isNotBlank(visitInfoDto.getIdCard())) {
                    idCard = Arrays.asList(visitInfoDto.getIdCard().split(",|，")).stream()
                            .filter(p -> StringUtils.isNotEmpty(p)).collect(Collectors.toList());
                }
                List<String> visitName = Arrays.asList(visitInfoDto.getVisitName().split(",|，")).stream()
                        .filter(u -> StringUtils.isNotEmpty(u)).collect(Collectors.toList());
                for (int i = 0; i < visitName.size(); i++) {
                    // 判断当前访客(用户)是否已经存在
                    SysUser sysUser = new SysUser();
                    sysUser.setName(visitName.get(i));
                    sysUser.setPhone(visitPhone.get(i));
                    sysUser.setUsername(visitPhone.get(i));
                    sysUser.setNormal();
                    List<SysUser> sysUserList = sysUserRepositorys.querySysUsers(sysUser);
                    // 不存在新增
                    if (!CollectionUtils.isEmpty(sysUserList)) {
                        if (sysUserList.get(0).getUserType().equals("0")) {
                            BeanUtil.copyProperties(visitInfoDto, errorDTO);
                            errorDTO.setError("员工不能作为访客!");
                            errorDTOArrayList.add(errorDTO);
                            continue;
                        }
                        if (sysUserList.get(0).getBlacklistFlag().toString().equals("1")) {
                            BeanUtil.copyProperties(visitInfoDto, errorDTO);
                            errorDTO.setError("不能邀请黑名单内的访客!");
                            errorDTOArrayList.add(errorDTO);
                            continue;
                        }
                    }
                    VisitUser visitUser = new VisitUser();
                    visitUser.setUserName(visitName.get(i));
                    visitUser.setVisitPhone(visitPhone.get(i));
                    visitUser.setType("1");
                    if (visitPhone.get(i).equals(visitInfoDto.getMainPhone()))
                        visitUser.setType("0");
                    if (idCard != null && !idCard.get(i).equals("空白"))
                        visitUser.setIdCard(idCard.get(i));
                    userList.add(visitUser);
                }
                visitInfoMation.setUserList(userList);
                // 新增
                if (visitInfoMation.getUserList().size() == visitPhone.size())
                    persist(visitInfoMation);
            }
        }
        if (null != errorDTOArrayList && errorDTOArrayList.size() > 0) {
            CyEpExcelUtil.exportExcel(errorDTOArrayList, "邀访错误数据", "邀访错误数据",
                    VisitInformationErrorDTO.class, "邀访错误数据.xlsx", response);
        }
        return null;
    }

    @Override
    public List<VisitInfoMation> findByUser(VisitUser visitUser) {
        return baseMapper.findByUser(visitUser);
    }

    public void exportTxtPaper(List<Map<Integer, List<String>>> list, String filename) throws Exception {
        String basePath = baseUploadLocation.substring(0, baseUploadLocation.lastIndexOf("/") - 1);
        String path = basePath + tempLocation + "/";
        File file = new File(path);
        File file1 = new File(path + filename);
        if (!file.isDirectory()) {
            file.mkdirs();
        }
        if (!file1.exists()) {
            boolean boo = file1.createNewFile();
            if (!boo) {
                log.info("文件创建失败");
            }
        }
        BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file1, false), "utf-8"), 10240);
        try {
            Iterator var8 = list.iterator();
            while (var8.hasNext()) {
                Map<Integer, List<String>> map = (Map) var8.next();
                Iterator var10 = map.keySet().iterator();
                while (var10.hasNext()) {
                    Integer s = (Integer) var10.next();
                    List<String> list1 = (List) map.get(s);
                    if (s != -1) {
                        out.write("第" + s + "行");
                    }
                    for (int i = 0; i < list1.size(); ++i) {
                        Object var10001 = list1.get(i);
                        out.write((String) var10001 + "\r\n");
                        out.write("       ");
                    }
                    out.newLine();
                }
            }
        } catch (Exception var22) {
            log.info("context:" + var22);
        } finally {
            try {
                out.flush();
                out.close();
            } catch (IOException var21) {
                log.info("context:" + var21);
            }
        }
    }


    /**
     * 下载错误文件
     *
     * @param response
     */
    @Override
    public int download(String fileName, HttpServletResponse response) {
        String basePath = baseUploadLocation.substring(0, baseUploadLocation.lastIndexOf("/") - 1);
        String path = basePath + tempLocation + "/" + fileName;
        response.setCharacterEncoding("UTF-8");
        response.setHeader("Content-Type", "application/x-msdownload");
        response.setHeader("Content-Disposition",
                "attachment;filename=" + fileName);
        File file = null;
        ServletOutputStream os = null;
        try {
            file = new File(path);
            os = response.getOutputStream();
            byte bytes[] = FileUtils.readFileToByteArray(file);//read文件成字节数组，然后将字节数组写入os输出流中去
            os.write(bytes);
            os.flush();
            os.close();
        } catch (Exception e) {
            log.info("context:" + e);
        }
        return 1;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public CyPersistModel persistWX(VisitInfoMation visitInfomation) {
        //增加操作
        if (CollectionUtils.isEmpty(visitInfomation.getUserList()))
            throw new CyServiceException(500, "请添加主访客信息");
        visitInfomation.setNormal();
        String[] regions = visitInfomation.getVisitRegions();
        String region = "";
        if (regions != null) {
            for (String r : regions) {
                region = region + r + ',';
            }
            if (region.length() > 0) {
                region = region.substring(0, region.length() - 1);
                visitInfomation.setVisitRegion(region);
            }
        }
        String[] regionCodes = visitInfomation.getVisitRegionCodes();
        String regionCode = "";
        if (regionCodes != null) {
            for (String code : regionCodes) {
                regionCode = regionCode + code + ",";
            }
            if (regionCode.length() > 0) {
                regionCode = regionCode.substring(0, regionCode.length() - 1);
                visitInfomation.setVisitRegionCode(regionCode);
            }
        }
        visitInfomation.setItemId(UUID.randomUUID().toString());
        int line = baseMapper.insert(visitInfomation);
        log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                visitInfomation.getBusinessId() + "的邀访信息");
        // 添加访客信息
        visitInfomation.getUserList().forEach(u -> {
            // 判断当前访客(用户)是否已经存在
            SysUser sysUser = new SysUser();
            sysUser.setName(u.getUserName());
            sysUser.setPhone(u.getVisitPhone());
            sysUser.setUsername(u.getVisitPhone());
            sysUser.setIdNumber(u.getIdCard());
            sysUser.setNormal();
            List<SysUser> sysUserList = sysUserRepositorys.querySysUsersByPhone(u.getVisitPhone());
            // 不存在新增
            if (CollectionUtils.isEmpty(sysUserList)) {
                sysUserRepositorys.insert(sysUser);
                u.setPersonId(UUID.randomUUID().toString());
                u.setUserId(sysUser.getBusinessId());
            } else {
                // 存在覆盖
                if (sysUserList.get(0).getUserType().equals("0")) {
                    throw new CyServiceException(500, "手机号已被员工绑定，请检查!");
                }
                if (sysUserList.get(0).getUserType().equals("1")) {
                    if (sysUserList.get(0).getBlacklistFlag().toString().equals("1")) {
                        throw new CyServiceException(500, "不能邀请黑名单内的访客!");
                    }
                    // 判断是否修改
                    boolean updateFlag = false;
                    if (!sysUserList.get(0).getName().equals(u.getUserName())) {
                        sysUserList.get(0).setName(u.getUserName());
                        updateFlag = true;
                    }
                    // 用户表里身份证为空 或 身份证号不同 进行覆盖
                    if (StringUtils.isEmpty(sysUserList.get(0).getIdNumber())|| (StringUtils.isNotEmpty(u.getIdCard()) && !sysUserList.get(0).getIdNumber().equals(u.getIdCard()))) {
                        sysUserList.get(0).setIdNumber(u.getIdCard());
                        updateFlag = true;
                    }
                    if (updateFlag) {
                        sysUserRepositorys.updateSysUser(sysUserList.get(0));
                    }
                    u.setPersonId(UUID.randomUUID().toString());
                    u.setUserId(sysUserList.get(0).getBusinessId());
                }
            }
            u.setVisitId(visitInfomation.getBusinessId());
            u.setNormal();
            visitUserRepository.insert(u);
            if ("0".equals(u.getType())) {
                //1.邀请人提交表单给主访客发送信息
                //给主访客发站内信：
                MessageInstance messageInstance = new MessageInstance();
                messageInstance.setTitle("访问邀请信息");
                messageInstance.setContent("您收到一条来自国药现代制药" + visitInfomation.getUserName() + "的邀请，请尽快补充访问信息 " +
                        "<a href=\"/pages/init/init?flag=0&id=" + visitInfomation.getBusinessId() + "\" style=\"color: #3497CE;\">点击查看</a>");
                messageInstance.setRecipientId(BigInteger.valueOf(u.getUserId()));
                messageInstanceService.sendMessage(messageInstance);
                if (message) {
                    String token = cyWxMiniServiceImpl.getAccessToken();
                    //公众号推送消息
                    Map<String, Object> map = new HashMap<>();
                    Map<String, String> keyMap = new HashMap<>();
                    keyMap.put("value", "您收到一条来自国药现代制药" + visitInfomation.getUserName() + "的邀请，请尽快补充访问信息");
                    keyMap.put("color", "#173177");
                    map.put("first", keyMap);
                    keyMap = new HashMap<>();
                    keyMap.put("value", u.getUserName());
                    keyMap.put("color", "#173177");
                    map.put("keyword1", keyMap);
                    keyMap = new HashMap<>();
                    keyMap.put("value", u.getVisitPhone());
                    keyMap.put("color", "#173177");
                    map.put("keyword2", keyMap);
                    keyMap = new HashMap<>();
                    if (!"其他".equals(visitInfomation.getVisitReason()))
                        keyMap.put("value", visitInfomation.getVisitReason());
                    else
                        keyMap.put("value", visitInfomation.getVisitReasonOther());
                    keyMap.put("color", "#173177");
                    map.put("keyword3", keyMap);
                    keyMap = new HashMap<>();
                    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
                    if (visitInfomation.getVisitRealTime() != null)
                        keyMap.put("value", format.format(visitInfomation.getVisitRealTime()));
                    else
                        keyMap.put("value", "");
                    keyMap.put("color", "#173177");
                    map.put("keyword4", keyMap);
                    SysUser user = sysUserRepositorys.getUserByBusinessId(u.getUserId());
                    String path = "/subpkgVisitors/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                    if (user != null)
                        cyWxMiniServiceImpl.pushWeChatMessage(token, user.getWxOpenid(), inviteTemplateId, path, map);
                }
            }
        });
        return new CyPersistModel(line,"新增邀访数据",visitInfomation);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public CyPersistModel check(VisitInfoMation visitInfomation, MessageInstance messageInstance,String wxToken) {
        int line = 0;
        if (visitInfomation.getUpdateType().equals("1")) {
            // 判断当前时间是否已经超过访问时间8小时
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            Date date = null;
            try {
                Calendar cal = Calendar.getInstance();
                cal.setTime(visitInfomation.getVisitRealTime());
                cal.add(Calendar.HOUR, 8);// 24小时制
                date = cal.getTime();
                if (format.parse(format.format(new Date())).compareTo(format.parse(format.format(date))) > 0) {
                    throw new CyServiceException(500, "当前表单已失效");
                }
            } catch (Exception ex) {
                throw new CyServiceException(500, "当前表单已失效");
            }
            visitInfomation.setState("2");
            visitInfomation.setVisitStatus("1");
            //2.主访客“提交审核”给邀请的员工发送通知
            //给邀请人发送站内信
            VisitInfoMation inf = baseMapper.selectById(visitInfomation.getBusinessId());
            messageInstance.setTitle("访问信息审核");
            messageInstance.setContent("您收到一条来自访客的访问审核，请在" + format.format(visitInfomation.getVisitRealTime()) + "前进行审核 " +
                    "<a href=\"/pages/init/init?flag=2&id=" + visitInfomation.getBusinessId() + "\" style=\"color: #3497CE;\">点击查看</a>");
            messageInstance.setRecipientId(BigInteger.valueOf(inf.getUserId()));
            messageInstanceService.sendMessage(messageInstance);
            List<VisitUser> users = baseMapper.getUserList(visitInfomation.getBusinessId());
            String userNames = users.stream().map(VisitUser::getUserName).collect(Collectors.joining(","));
            String visitPhone = users.stream().map(VisitUser::getVisitPhone).collect(Collectors.joining(","));
            if (message) {
                String token = cyWxMiniServiceImpl.getAccessToken();
                //公众号推送消息
                Map<String, Object> map = new HashMap<>();
                Map<String, String> keyMap = new HashMap<>();
                keyMap.put("value", "您收到一条来自访客的访问审核，请在" + format.format(visitInfomation.getVisitRealTime()) + "前进行审核");
                keyMap.put("color", "#173177");
                map.put("first", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("value", "国药现代制药");
                keyMap.put("color", "#173177");
                map.put("keyword1", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                keyMap.put("value", format.format(inf.getVisitRealTime()));
                map.put("keyword2", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                keyMap.put("value", userNames);
                map.put("keyword3", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                keyMap.put("value", visitPhone);
                map.put("keyword4", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                if (!"其他".equals(inf.getVisitReason()))
                    keyMap.put("value", inf.getVisitReason());
                else
                    keyMap.put("value", inf.getVisitReasonOther());
                map.put("keyword5", keyMap);
                SysUser user = sysUserRepositorys.getUserByBusinessId(inf.getUserId());
                String path = "/subpkg/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                cyWxMiniServiceImpl.pushWeChatMessage(token, user.getWxOpenid(), examineTemplateId, path, map);
            }
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            visitInfomation.setState("5");
            visitInfomation.setVisitStatus("2");
            //给所有访客发送站内信：您计划于（访问时间，日期时分）到访国药现代制药的申请审核通过/未通过，点击查看
            if (StringUtils.isNotBlank(messageInstance.getUserIds())) {
                for (String userId : messageInstance.getUserIds().split(",")) {
                    messageInstance.setRecipientId(BigInteger.valueOf(Long.parseLong(userId)));
                    messageInstanceService.sendMessage(messageInstance);
                }
            }
            if (messageInstance.getUsers() != null && !messageInstance.getUsers().isEmpty()){
                for (VisitUser user : messageInstance.getUsers()) {
                    messageInstance.setRecipientId(BigInteger.valueOf(user.getUserId()));
                    messageInstanceService.sendMessage(messageInstance);
                    if (message) {
                        //公众号推送消息
                        Map<String, Object> map = new HashMap<>();
                        Map<String, String> keyMap = new HashMap<>();
                        keyMap.put("value", "您计划于" + format.format(visitInfomation.getVisitRealTime()) +
                                "到访国药现代制药的申请因邀请人超时未审核已失效");
                        keyMap.put("color", "#173177");
                        map.put("first", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("value", user.getUserName());
                        keyMap.put("color", "#173177");
                        map.put("keyword1", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("color", "#173177");
                        keyMap.put("value", format.format(visitInfomation.getVisitRealTime()));
                        map.put("keyword2", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("color", "#173177");
                        if (!"其他".equals(visitInfomation.getVisitReason()))
                            keyMap.put("value", visitInfomation.getVisitReason());
                        else
                            keyMap.put("value", visitInfomation.getVisitReasonOther());
                        map.put("keyword3", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("color", "#173177");
                        keyMap.put("value", "超时未审核");
                        map.put("keyword4", keyMap);
                        String path = "/subpkgVisitors/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                        cyWxMiniServiceImpl.pushWeChatMessage(wxToken, user.getWxOpenid(), resultTemplateId, path, map);
                    }
                }
            }
            if (messageInstance.getResultUsers()!=null && !messageInstance.getResultUsers().isEmpty()){
                VisitInfoMation inf = baseMapper.selectById(visitInfomation.getBusinessId());
                for (VisitUser user : messageInstance.getResultUsers()) {
                    messageInstance.setRecipientId(BigInteger.valueOf(user.getUserId()));
                    messageInstanceService.sendMessage(messageInstance);
                    if (message) {
                        //公众号推送消息
                        Map<String, Object> map = new HashMap<>();
                        Map<String, String> keyMap = new HashMap<>();
                        keyMap.put("value", "您计划于" + format.format(inf.getVisitRealTime()) + "到访国药现代制药的申请审核未通过");
                        keyMap.put("color", "#173177");
                        map.put("first", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("value", user.getUserName());
                        keyMap.put("color", "#173177");
                        map.put("keyword1", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("color", "#173177");
                        keyMap.put("value", format.format(inf.getVisitRealTime()));
                        map.put("keyword2", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("color", "#173177");
                        if (!"其他".equals(inf.getVisitReason()))
                            keyMap.put("value", inf.getVisitReason());
                        else
                            keyMap.put("value", inf.getVisitReasonOther());
                        map.put("keyword3", keyMap);
                        keyMap = new HashMap<>();
                        keyMap.put("color", "#173177");
                        keyMap.put("value", "审核未通过");
                        map.put("keyword4", keyMap);
                        String path = "/subpkgVisitors/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                        cyWxMiniServiceImpl.pushWeChatMessage(wxToken, user.getWxOpenid(), resultTemplateId, path, map);
                    }
                }
            }
        }
        // 修改邀访信息以及访客状态状态为待审核
        //UpdateWrapper更新操作
        UpdateWrapper<VisitInfoMation> warp = new UpdateWrapper<>();
        //通过set设置需要修改的内容，eq设置条件
        warp.set("visit_status", visitInfomation.getVisitStatus()).eq("business_id", visitInfomation.getBusinessId());
        line = baseMapper.update(null, warp);
        line = line * visitUserRepository.updateByVisitId(visitInfomation.getState(), visitInfomation.getBusinessId());
        return new CyPersistModel(line);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    public CyPersistModel examinePass(VisitInfoMation visitInfomation) {
        int line;
        VisitInfoMation v = baseMapper.queryVisitById(visitInfomation.getBusinessId());
        UpdateWrapper<VisitUser> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("state", "3").eq("visit_id", visitInfomation.getBusinessId());
        line = visitUserRepository.update(null, updateWrapper);
        UpdateWrapper<VisitInfoMation> warp = new UpdateWrapper<>();
        warp.set("visit_status", "3");
        visitInfomation.setItemId(v.getItemId());
        visitInfomation.setUserList(v.getUserList());
        visitInfomation.setVisitReasonCode(v.getVisitReasonCode());
        if (synchronization && jieLinkService.pushVisitorData(visitInfomation)) {
            warp.set("syn_status", "1");
        }
        warp.eq("business_id", visitInfomation.getBusinessId());
        line = line * baseMapper.update(null, warp);
        //给所有访客发送站内信：您计划于（访问时间，日期时分）到访国药现代制药的申请审核通过/未通过，点击查看
        MessageInstance messageInstance = new MessageInstance();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        String token = cyWxMiniServiceImpl.getAccessToken();
        for (VisitUser visitUser : v.getUserList()) {
            VisitInfoMation inf = baseMapper.selectById(visitInfomation.getBusinessId());
            //3.员工通过审核给所有访客发送消息
            messageInstance = new MessageInstance();
            messageInstance.setTitle("访问审核结果");
            messageInstance.setContent("您计划于" + format.format(inf.getVisitRealTime()) + "到访国药现代制药的申请审核通过 " +
                    "<a href=\"/pages/init/init?flag=1&id=" + visitInfomation.getBusinessId() + "\" style=\"color: #3497CE;\">点击查看</a>");
            messageInstance.setRecipientId(BigInteger.valueOf(visitUser.getUserId()));
            messageInstanceService.sendMessage(messageInstance);
            if (message) {
                //公众号推送消息
                Map<String, Object> map = new HashMap<>();
                Map<String, String> keyMap = new HashMap<>();
                keyMap.put("value", "您计划于" + format.format(inf.getVisitRealTime()) + "到访国药现代制药的申请审核通过");
                keyMap.put("color", "#173177");
                map.put("first", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("value", visitUser.getUserName());
                keyMap.put("color", "#173177");
                map.put("keyword1", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                keyMap.put("value", format.format(inf.getVisitRealTime()));
                map.put("keyword2", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                if (!"其他".equals(v.getVisitReason()))
                    keyMap.put("value", v.getVisitReason());
                else
                    keyMap.put("value", v.getVisitReasonOther());
                map.put("keyword3", keyMap);
                keyMap = new HashMap<>();
                keyMap.put("color", "#173177");
                keyMap.put("value", "审核通过");
                map.put("keyword4", keyMap);
                String path = "/subpkgVisitors/messageDetails/messageDetails?id=" + messageInstance.getBusinessId();
                cyWxMiniServiceImpl.pushWeChatMessage(token, visitUser.getWxOpenid(), resultTemplateId, path, map);
            }
        }
        return new CyPersistModel(line);
    }


    void errorText(String error, ImportVistInfoDTO importVistInfoDTO, List<Map<Integer, List<String>>> maps, String fileName) {
        List<String> strings = new ArrayList<String>();
        strings.add(error);
        Map<Integer, List<String>> map = new HashMap<>();
        map.put(-1, strings);
        maps.add(map);
        importVistInfoDTO.setFilename(fileName);
    }
}
